@@ -13,8 +13,8 @@
  *		See the LICENSE file at the top of the source tree.
  * \warning     File has been Lined up using 8 Space TABS
  *
- * $Date: 2014-05-02 09:48:10 +0200 (Fri, 02 May 2014) $
- * $Revision: 5398 $
+ * $Date: 2014-06-22 12:58:33 +0200 (Sun, 22 Jun 2014) $
+ * $Revision: 5502 $
  */
 
 #ifndef __CHAN_SCCP_H
@@ -34,8 +34,11 @@ extern "C" {
 #include <config.h>
 //#include "common.h"
 
+#define ENUMMACRO_FILE "chan_sccp_enums.hh"
+#define ENUMMACRO_GENERATE ENUMMACRO_GENHEADER
 #include "sccp_enum_macro.h"
-#include "chan_sccp_enums.hh"
+#undef  ENUMMACRO_GENERATE
+#undef  ENUMMACRO_FILE
 
 #define sccp_mutex_t ast_mutex_t
 
@@ -131,7 +134,8 @@ char SCCP_REVISIONSTR[30];
 #define sccp_false(x) (pbx_false(x) ? 1 : 0)
 
 // When DEBUGCAT_HIGH is set, we use ast_log instead of ast_verbose
-#define sccp_log1(...) { if ((sccp_globals->debug & (DEBUGCAT_FILELINEFUNC)) == DEBUGCAT_FILELINEFUNC) { ast_log(AST_LOG_NOTICE, __VA_ARGS__); } else { ast_verbose(__VA_ARGS__); } }
+//#define sccp_log1(...) { if ((sccp_globals->debug & (DEBUGCAT_FILELINEFUNC)) == DEBUGCAT_FILELINEFUNC) { ast_log(AST_LOG_NOTICE, __VA_ARGS__); } else { ast_verbose(__VA_ARGS__); } }
+#define sccp_log1(...) { ast_log(AST_LOG_DEBUG, __VA_ARGS__); ast_verbose(__VA_ARGS__); }
 #define sccp_log(_x) if ((sccp_globals->debug & (_x))) sccp_log1
 #define sccp_log_and(_x) if ((sccp_globals->debug & (_x)) == (_x)) sccp_log1
 
@@ -872,6 +876,7 @@ struct sccp_line {
 	char language[SCCP_MAX_LANGUAGE];									/*!< language we use for calls */
 	char accountcode[SCCP_MAX_ACCOUNT_CODE];								/*!< accountcode used in cdr */
 	char musicclass[SCCP_MAX_MUSICCLASS];									/*!< musicclass assigned when getting moh */
+	char parkinglot[SCCP_MAX_CONTEXT];									/*!< parkinglot to use */
 };														/*!< SCCP Line Structure */
 
 /*!
@@ -899,7 +904,7 @@ struct sccp_device {
 	uint32_t skinny_type;											/*!< Model of this Phone sent by the station, devicetype */
 	uint32_t device_features;										/*!< device features (contains protocolversion in 8bit first segement */
 	sccp_devicestate_t state;										/*!< Device State (SCCP_DEVICE_ONHOOK or SCCP_DEVICE_OFFHOOK) */
-	sccp_channelstate_t earlyrtp;										/*!< RTP Channel State where to open the RTP Media Stream */
+	sccp_earlyrtp_t earlyrtp;										/*!< RTP Channel State where to open the RTP Media Stream */
 	sccp_session_t *session;										/*!< Current Session */
 	SCCP_RWLIST_ENTRY (sccp_device_t) list;									/*!< Global Device Linked List */
 	uint16_t keepalive;												/*!< Station Specific Keepalive Timeout */
@@ -1199,6 +1204,8 @@ struct sccp_channel {
 	struct {
 		struct sccp_rtp audio;										/*!< Asterisk RTP */
 		struct sccp_rtp video;										/*!< Video RTP session */
+		//struct sccp_rtp text;										/*!< Video RTP session */
+		uint8_t peer_dtmfmode;
 	} rtp;
 
 	uint16_t ringermode;											/*!< Ringer Mode */
@@ -1226,6 +1233,7 @@ struct sccp_channel {
 	/* next should be converted to pointers, to reduce size */
 	char musicclass[SCCP_MAX_MUSICCLASS];									/*!< Music Class */
 	sccp_callinfo_t callInfo;
+	sccp_dtmfmode_t dtmfmode;										/*!< DTMF Mode (0 inband - 1 outofband) */
 };														/*!< SCCP Channel Structure */
 
 /*!
@@ -1296,7 +1304,7 @@ struct sccp_global_vars {
 	boolean_t echocancel;											/*!< Echo Canel Support (Boolean, default=on) */
 	boolean_t silencesuppression;										/*!< Silence Suppression Support (Boolean, default=on)  */
 	boolean_t trustphoneip;											/*!< Trust Phone IP Support (Boolean, default=on) */
-	sccp_channelstate_t earlyrtp;										/*!< Channel State where to open the rtp media stream */
+	sccp_earlyrtp_t earlyrtp;										/*!< Channel State where to open the rtp media stream */
 	boolean_t privacy;											/*!< Privacy Support (Length=2) */
 	skinny_lampmode_t mwilamp;										/*!< MWI/Lamp (Length:3) */
 	boolean_t mwioncall;											/*!< MWI On Call Support (Boolean, default=on) */
