@@ -8,8 +8,8 @@
  * \note	This program is free software and may be modified and distributed under the terms of the GNU Public License.
  *		See the LICENSE file at the top of the source tree.
  *
- * $Date: 2010-10-23 20:04:30 +0200 (Sat, 23 Oct 2010) $
- * $Revision: 2044 $  
+ * $Date: 2014-05-30 19:56:13 +0200 (Fri, 30 May 2014) $
+ * $Revision: 5449 $  
  */
 #ifndef __SCCP_PBX_WRAPPER_H
 #define __SCCP_PBX_WRAPPER_H
@@ -355,12 +355,8 @@ sccp_channel_t *__get_sccp_channel_from_pbx_channel(const PBX_CHANNEL_TYPE * pbx
 sccp_channel_t *get_sccp_channel_from_pbx_channel(const PBX_CHANNEL_TYPE * pbx_channel);
 #endif
 int sccp_asterisk_pbx_fktChannelWrite(PBX_CHANNEL_TYPE * ast, const char *funcname, char *args, const char *value);
-#ifdef CS_EXPERIMENTAL
 boolean_t sccp_wrapper_asterisk_requestQueueHangup(sccp_channel_t *channel);
 boolean_t sccp_wrapper_asterisk_requestHangup(sccp_channel_t *channel);
-#else
-int sccp_wrapper_asterisk_requestHangup(PBX_CHANNEL_TYPE * pbx_channel);
-#endif
 
 /***** database *****/
 boolean_t sccp_asterisk_addToDatabase(const char *family, const char *key, const char *value);
