@@ -8,8 +8,8 @@
  * \note	This program is free software and may be modified and distributed under the terms of the GNU Public License.
  *		See the LICENSE file at the top of the source tree.
  *
- * $Date: 2010-10-23 20:04:30 +0200 (Sat, 23 Oct 2010) $
- * $Revision: 2044 $  
+ * $Date: 2014-05-30 19:56:13 +0200 (Fri, 30 May 2014) $
+ * $Revision: 5449 $  
  */
 #ifndef __PBX_IMPL_H
 #define __PBX_IMPL_H
@@ -40,9 +40,6 @@ struct sccp_pbx_cb {
 	int (*const set_callstate) (const sccp_channel_t * pbx_channel, int state);
 	boolean_t(*const checkhangup) (const sccp_channel_t * channel);
 	int (*const hangup) (PBX_CHANNEL_TYPE * channel);
-#ifndef CS_EXPERIMENTAL
-	int (*const requestHangup) (PBX_CHANNEL_TYPE * channel);
-#endif 
 	sccp_extension_status_t(*const extension_status) (const sccp_channel_t * channel);
 	
 	void (*const setPBXChannelLinkedId) (PBX_CHANNEL_TYPE *pbxchannel, const char *linkedid);
