@@ -9,8 +9,8 @@
  * \note	This program is free software and may be modified and distributed under the terms of the GNU Public License.
  *		See the LICENSE file at the top of the source tree.
  *
- * $Date: 2014-05-02 09:48:10 +0200 (Fri, 02 May 2014) $
- * $Revision: 5398 $  
+ * $Date: 2014-06-06 14:25:45 +0200 (Fri, 06 Jun 2014) $
+ * $Revision: 5475 $  
  */
 
 #ifndef __SCCP_CHANNEL_H
@@ -28,7 +28,7 @@ void sccp_channel_updateChannelDesignator(sccp_channel_t * c);
 void sccp_channel_updateChannelCapability(sccp_channel_t * channel);
 void sccp_channel_send_callinfo(const sccp_device_t * device, const sccp_channel_t * c);
 void sccp_channel_send_callinfo2(sccp_channel_t * c);
-void sccp_channel_setSkinnyCallstate(sccp_channel_t * c, skinny_callstate_t state);
+void sccp_channel_setChannelstate(sccp_channel_t * channel, sccp_channelstate_t state);
 void sccp_channel_display_callInfo(sccp_channel_t * channel);
 void sccp_channel_set_callingparty(sccp_channel_t * c, char *name, char *number);
 void sccp_channel_set_calledparty(sccp_channel_t * c, char *name, char *number);
