@@ -15,8 +15,8 @@
  * Relations:	SCCP Device -> SCCP DeviceLine -> SCCP Line
  * 		SCCP Line -> SCCP ButtonConfig -> SCCP Device
  *
- * \date	$Date: 2014-05-02 09:48:10 +0200 (Fri, 02 May 2014) $
- * \version     $Revision: 5398 $
+ * \date	$Date: 2014-06-27 00:32:10 +0200 (Fri, 27 Jun 2014) $
+ * \version     $Revision: 5526 $
  */
 
 #include <config.h>
@@ -32,7 +32,7 @@
 #include "sccp_indicate.h"
 #include "sccp_mwi.h"
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 5398 $")
+SCCP_FILE_VERSION(__FILE__, "$Revision: 5526 $")
 int __sccp_device_destroy(const void *ptr);
 void sccp_device_removeFromGlobals(sccp_device_t * device);
 int sccp_device_destroy(const void *ptr);
@@ -41,7 +41,7 @@ int sccp_device_destroy(const void *ptr);
 static void sccp_device_old_indicate_remoteHold(const sccp_device_t * device, uint8_t lineInstance, uint8_t callid, uint8_t callpriority, uint8_t callPrivacy);
 static void sccp_device_new_indicate_remoteHold(const sccp_device_t * device, uint8_t lineInstance, uint8_t callid, uint8_t callpriority, uint8_t callPrivacy);
 static void sccp_device_indicate_offhook(const sccp_device_t * device, sccp_linedevices_t * linedevice, uint8_t callid);
-static void sccp_device_indicate_onhook(const sccp_device_t * device, const uint8_t lineInstance, uint8_t callid);
+static void sccp_device_indicate_onhook(const sccp_device_t * device, const sccp_channel_t *channel, const uint8_t lineInstance, uint8_t callid);
 static void sccp_device_indicate_dialing(const sccp_device_t * device, const uint8_t lineInstance, const sccp_channel_t * channel);
 static void sccp_device_indicate_proceed(const sccp_device_t * device, const uint8_t lineInstance, const sccp_channel_t * channel);
 static void sccp_device_indicate_connected(const sccp_device_t * device, sccp_linedevices_t * linedevice, const sccp_channel_t * channel);
@@ -282,8 +282,6 @@ void sccp_device_pre_reload(void)
 	SCCP_RWLIST_WRLOCK(&GLOB(devices));
 	SCCP_RWLIST_TRAVERSE(&GLOB(devices), d, list) {
 		sccp_log((DEBUGCAT_CONFIG + DEBUGCAT_DEVICE)) (VERBOSE_PREFIX_2 "%s: Setting Device to Pending Delete=1\n", d->id);
-		sccp_free_ha(d->ha);
-		d->ha = NULL;
 		if (!d->realtime) {										/* don't want to reset hotline devices. */
 			d->pendingDelete = 1;
 		}
@@ -309,10 +307,10 @@ void sccp_device_pre_reload(void)
  */
 boolean_t sccp_device_check_update(sccp_device_t * device)
 {
-	sccp_device_t *d = NULL;
+	AUTO_RELEASE sccp_device_t *d = sccp_device_retain(device);
 	boolean_t res = FALSE;
 
-	if ((d = sccp_device_retain(device))) {
+	if (d) {
 		sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_1 "device: %s check_update, pendingUpdate: %s, pendingDelete: %s\n", d->id, d->pendingUpdate ? "TRUE" : "FALSE", d->pendingDelete ? "TRUE" : "FALSE");
 		if ((d->pendingUpdate || d->pendingDelete)) {
 			do {
@@ -350,7 +348,6 @@ boolean_t sccp_device_check_update(sccp_device_t * device)
 				res = TRUE;
 			} while (0);
 		}
-		d = sccp_device_release(d);
 	}
 	return res;
 }
@@ -481,6 +478,7 @@ sccp_device_t *sccp_device_createAnonymous(const char *name)
 
 void sccp_device_setLastNumberDialed(sccp_device_t * device, const char *lastNumberDialed)
 {
+	boolean_t ResetNoneLineInstance = FALSE;
 #ifdef CS_ADV_FEATURES
     	if (device->useRedialMenu) {
     		return;
@@ -490,6 +488,9 @@ void sccp_device_setLastNumberDialed(sccp_device_t * device, const char *lastNum
 	
 	sccp_log(DEBUGCAT_DEVICE) (VERBOSE_PREFIX_3 "%s: Update last number dialed to %s.\n", DEV_ID_LOG(device), lastNumberDialed);
 	if (lastNumberDialed && !sccp_strlen_zero(lastNumberDialed)) {
+		if (sccp_strlen_zero(device->lastNumber)) {
+			ResetNoneLineInstance = TRUE;
+		}
 		sccp_copy_string(device->lastNumber, lastNumberDialed, sizeof(device->lastNumber));
 		redial_active = TRUE;
 	} else {
@@ -500,6 +501,9 @@ void sccp_device_setLastNumberDialed(sccp_device_t * device, const char *lastNum
 	sccp_softkey_setSoftkeyState(device, KEYMODE_OFFHOOK, SKINNY_LBL_REDIAL, redial_active);
 	sccp_softkey_setSoftkeyState(device, KEYMODE_OFFHOOKFEAT, SKINNY_LBL_REDIAL, redial_active);
 	sccp_softkey_setSoftkeyState(device, KEYMODE_ONHOOKSTEALABLE, SKINNY_LBL_REDIAL, redial_active);
+	if (ResetNoneLineInstance) {
+		sccp_dev_set_keyset(device, 0, 0, KEYMODE_ONHOOK);
+	}
 }
 
 /*!
@@ -602,7 +606,7 @@ void sccp_dev_build_buttontemplate(sccp_device_t * d, btnlist * btn)
 {
 	uint8_t i;
 
-	sccp_log((DEBUGCAT_CONFIG + DEBUGCAT_BUTTONTEMPLATE + DEBUGCAT_DEVICE)) (VERBOSE_PREFIX_3 "%s: Building button template %s(%d), user config %s\n", d->id, devicetype2str(d->skinny_type), d->skinny_type, d->config_type);
+	sccp_log((DEBUGCAT_CONFIG + DEBUGCAT_BUTTONTEMPLATE + DEBUGCAT_DEVICE)) (VERBOSE_PREFIX_3 "%s: Building button template %s(%d), user config %s\n", d->id, skinny_devicetype2str(d->skinny_type), d->skinny_type, d->config_type);
 
 	switch (d->skinny_type) {
 		case SKINNY_DEVICETYPE_30SPPLUS:
@@ -927,13 +931,15 @@ void sccp_dev_sendmsg(const sccp_device_t * d, sccp_mid_t t)
  * \brief Register a Device
  * \param d SCCP Device
  * \param opt Option/Registration State as int
+ *
+ * \note adds a retained device to the event.deviceRegistered.device
  */
 void sccp_dev_set_registered(sccp_device_t * d, uint8_t opt)
 {
 	sccp_event_t event;
 	sccp_msg_t *msg;
 
-	sccp_log((DEBUGCAT_DEVICE)) (VERBOSE_PREFIX_3 "%s: (sccp_dev_set_registered) Setting Registered Status for Device from %s to %s\n", DEV_ID_LOG(d), registrationstate2str(d->registrationState), registrationstate2str(opt));
+	sccp_log((DEBUGCAT_DEVICE)) (VERBOSE_PREFIX_3 "%s: (sccp_dev_set_registered) Setting Registered Status for Device from %s to %s\n", DEV_ID_LOG(d), skinny_registrationstate2str(d->registrationState), skinny_registrationstate2str(opt));
 
 	if (d->registrationState == opt) {
 		return;
@@ -1032,7 +1038,7 @@ void sccp_dev_set_keyset(const sccp_device_t * d, uint8_t lineInstance, uint32_t
 	//msg->data.SelectSoftKeysMessage.les_validKeyMask = 0xFFFFFFFF;           /* htolel(65535); */
 	msg->data.SelectSoftKeysMessage.les_validKeyMask = htolel(d->softKeyConfiguration.activeMask[softKeySetIndex]);
 
-	sccp_log((DEBUGCAT_SOFTKEY + DEBUGCAT_DEVICE)) (VERBOSE_PREFIX_3 "%s: Send softkeyset to %s(%d) on line %d  and call %d\n", d->id, keymode2str(softKeySetIndex), softKeySetIndex, lineInstance, callid);
+	sccp_log((DEBUGCAT_SOFTKEY + DEBUGCAT_DEVICE)) (VERBOSE_PREFIX_3 "%s: Send softkeyset to %s(%d) on line %d  and call %d\n", d->id, skinny_keymode2str(softKeySetIndex), softKeySetIndex, lineInstance, callid);
 	sccp_log((DEBUGCAT_SOFTKEY + DEBUGCAT_DEVICE)) (VERBOSE_PREFIX_3 "%s: validKeyMask %u\n", d->id, msg->data.SelectSoftKeysMessage.les_validKeyMask);
 	sccp_dev_send(d, msg);
 }
@@ -1060,7 +1066,7 @@ void sccp_dev_set_ringer(const sccp_device_t * d, uint8_t opt, uint8_t lineInsta
 	msg->data.SetRingerMessage.lel_lineInstance = htolel(lineInstance);
 	msg->data.SetRingerMessage.lel_callReference = htolel(callid);
 	sccp_dev_send(d, msg);
-	sccp_log((DEBUGCAT_DEVICE)) (VERBOSE_PREFIX_3 "%s: Send ringer mode %s(%d) on device\n", DEV_ID_LOG(d), ringtype2str(opt), opt);
+	sccp_log((DEBUGCAT_DEVICE)) (VERBOSE_PREFIX_3 "%s: Send ringer mode %s(%d) on device\n", DEV_ID_LOG(d), skinny_ringtype2str(opt), opt);
 }
 
 /*!
@@ -1180,7 +1186,7 @@ void sccp_dev_starttone(const sccp_device_t * d, uint8_t tone, uint8_t line, uin
 	msg->data.StartToneMessage.lel_callReference = htolel(callid);
 
 	sccp_dev_send(d, msg);
-	sccp_log((DEBUGCAT_DEVICE)) (VERBOSE_PREFIX_3 "%s: Sending tone %s (%d)\n", d->id, tone2str(tone), tone);
+	sccp_log((DEBUGCAT_DEVICE)) (VERBOSE_PREFIX_3 "%s: Sending tone %s (%d) on line %d with callid %d\n", d->id, skinny_tone2str(tone), tone, line, callid);
 }
 
 /*!
@@ -1203,7 +1209,7 @@ void sccp_dev_stoptone(const sccp_device_t * d, uint8_t line, uint32_t callid)
 	msg->data.StopToneMessage.lel_lineInstance = htolel(line);
 	msg->data.StopToneMessage.lel_callReference = htolel(callid);
 	sccp_dev_send(d, msg);
-	sccp_log((DEBUGCAT_DEVICE)) (VERBOSE_PREFIX_3 "%s: Stop tone on device\n", d->id);
+	sccp_log((DEBUGCAT_DEVICE)) (VERBOSE_PREFIX_3 "%s: Stop tone on line %d with callid %d\n", d->id, line, callid);
 }
 
 /*!
@@ -1576,9 +1582,9 @@ sccp_channel_t *sccp_device_getActiveChannel(const sccp_device_t * d)
  */
 void sccp_device_setActiveChannel(sccp_device_t * d, sccp_channel_t * channel)
 {
-	sccp_device_t *device = NULL;
+	AUTO_RELEASE sccp_device_t *device = sccp_device_retain(d);
 
-	if ((device = sccp_device_retain(d))) {
+	if (device) {
 		sccp_log((DEBUGCAT_CHANNEL + DEBUGCAT_DEVICE)) (VERBOSE_PREFIX_3 "%s: Set the active channel %d on device\n", DEV_ID_LOG(d), (channel) ? channel->callid : 0);
 		if (device->active_channel && device->active_channel->line) {
 			device->active_channel->line->statistic.numberOfActiveChannels--;
@@ -1594,7 +1600,6 @@ void sccp_device_setActiveChannel(sccp_device_t * d, sccp_channel_t * channel)
 				device->active_channel->line->statistic.numberOfActiveChannels++;
 			}
 		}
-		device = sccp_device_release(device);
 	}
 }
 
@@ -1609,7 +1614,7 @@ void sccp_device_setActiveChannel(sccp_device_t * d, sccp_channel_t * channel)
  * \callgraph
  * \callergraph
  */
-void sccp_dev_check_displayprompt(sccp_device_t * d)
+void sccp_dev_check_displayprompt(const sccp_device_t * d)
 {
 	//sccp_log((DEBUGCAT_CORE + DEBUGCAT_DEVICE + DEBUGCAT_MESSAGE)) (VERBOSE_PREFIX_1 "%s: (sccp_dev_check_displayprompt)\n", DEV_ID_LOG(d));
 	if (!d || !d->session || !d->protocol || !d->hasDisplayPrompt()) {
@@ -1689,16 +1694,16 @@ void sccp_dev_forward_status(sccp_line_t * l, uint8_t lineInstance, sccp_device_
  * \param d SCCP Device
  * \return Result as int
  */
-int sccp_device_check_ringback(sccp_device_t * d)
+int sccp_device_check_ringback(sccp_device_t * device)
 {
-	sccp_channel_t *c;
+	AUTO_RELEASE sccp_channel_t *c;
+	AUTO_RELEASE sccp_device_t *d = sccp_device_retain(device);
 
-	if (!(d = sccp_device_retain(d))) {
+	if (!d) {
 		return 0;
 	}
 	d->needcheckringback = 0;
 	if (d->state == SCCP_DEVICESTATE_OFFHOOK) {
-		sccp_device_release(d);
 		return 0;
 	}
 	c = sccp_channel_find_bystate_on_device(d, SCCP_CHANNELSTATE_CALLTRANSFER);
@@ -1710,10 +1715,8 @@ int sccp_device_check_ringback(sccp_device_t * d)
 	}
 	if (c) {
 		sccp_indicate(d, c, SCCP_CHANNELSTATE_RINGING);
-		c = sccp_channel_release(c);
 		return 1;
 	}
-	sccp_device_release(d);
 	return 0;
 }
 
@@ -1724,6 +1727,7 @@ int sccp_device_check_ringback(sccp_device_t * d)
  * \callgraph
  * \callergraph
  * 
+ * \note adds a retained device to the event.deviceRegistered.device
  */
 void sccp_dev_postregistration(void *data)
 {
@@ -1738,7 +1742,6 @@ void sccp_dev_postregistration(void *data)
 	#endif
 	char family[ASTDB_FAMILY_KEY_LEN] = { 0 };
 	char buffer[ASTDB_RESULT_LEN] = { 0 };
-	sccp_linedevices_t *linedevice;
 	int instance;
 
 	if (!d) {
@@ -1754,7 +1757,8 @@ void sccp_dev_postregistration(void *data)
 	/* read last line/device states from db */
 	sccp_log((DEBUGCAT_DEVICE)) (VERBOSE_PREFIX_3 "%s: Getting Database Settings...\n", d->id);
 	for (instance = SCCP_FIRST_LINEINSTANCE; instance < d->lineButtons.size; instance++) {
-		if (d->lineButtons.instance[instance] && (linedevice = sccp_linedevice_retain(d->lineButtons.instance[instance]))) {
+		if (d->lineButtons.instance[instance]) {
+			AUTO_RELEASE sccp_linedevices_t *linedevice = sccp_linedevice_retain(d->lineButtons.instance[instance]);
 			sprintf(family, "SCCP/%s/%s", d->id, linedevice->line->name);
 			if (PBX(feature_getFromDatabase) (family, "cfwdAll", buffer, sizeof(buffer)) && strcmp(buffer, "")) {
 				linedevice->cfwdAll.enabled = TRUE;
@@ -1766,11 +1770,12 @@ void sccp_dev_postregistration(void *data)
 				sccp_copy_string(linedevice->cfwdBusy.number, buffer, sizeof(linedevice->cfwdAll.number));
 				sccp_feat_changed(d, linedevice, SCCP_FEATURE_CFWDBUSY);
 			}
-			sccp_linedevice_release(linedevice);
 		}
 	}
+	sprintf(family, "SCCP/%s", d->id);
 	if (PBX(feature_getFromDatabase) (family, "dnd", buffer, sizeof(buffer)) && strcmp(buffer, "")) {
 		sccp_config_parse_dnd(&d->dndFeature.status, sizeof(d->dndFeature.status), (const char *) buffer, SCCP_CONFIG_DEVICE_SEGMENT);
+		sccp_feat_changed(d, NULL, SCCP_FEATURE_DND);
 	}
 
 	if (PBX(feature_getFromDatabase) (family, "privacy", buffer, sizeof(buffer)) && strcmp(buffer, "")) {
@@ -1782,6 +1787,12 @@ void sccp_dev_postregistration(void *data)
                 sccp_feat_monitor(d, NULL, 0, NULL);
 		sccp_feat_changed(d, NULL, SCCP_FEATURE_MONITOR);
 	}
+	
+	char lastNumber[SCCP_MAX_EXTENSION] = "";
+	if (PBX(feature_getFromDatabase) (family, "lastDialedNumber", lastNumber, sizeof(lastNumber))) {
+		sccp_device_setLastNumberDialed(d, lastNumber);
+	}
+	
 
 	if (d->backgroundImage) {
 		d->setBackgroundImage(d, d->backgroundImage);
@@ -1790,6 +1801,11 @@ void sccp_dev_postregistration(void *data)
 	if (d->ringtone) {
 		d->setRingTone(d, d->ringtone);
 	}
+	
+	if (d->useRedialMenu && (!d->hasDisplayPrompt)) {
+		pbx_log(LOG_NOTICE, "%s: useRedialMenu is currently not supported on this devicetype. Reverting to old style redial\n", d->id);
+		d->useRedialMenu = FALSE;
+	}
 
 	sccp_dev_check_displayprompt(d);
 
@@ -1812,22 +1828,23 @@ void sccp_dev_postregistration(void *data)
  * \callgraph
  * \callergraph
  * 
+ * \note adds a retained device to the event.deviceRegistered.device
  */
-void sccp_dev_clean(sccp_device_t * d, boolean_t remove_from_global, uint8_t cleanupTime)
+void sccp_dev_clean(sccp_device_t * device, boolean_t remove_from_global, uint8_t cleanupTime)
 {
+	AUTO_RELEASE sccp_device_t *d =  sccp_device_retain(device);
 	sccp_buttonconfig_t *config = NULL;
 	sccp_selectedchannel_t *selectedChannel = NULL;
-	sccp_line_t *line = NULL;
-	sccp_channel_t *channel = NULL;
-	sccp_event_t event;
-	int i;
+	sccp_channel_t *c = NULL;
+	sccp_event_t event = { 0 };
+	int i = 0;
 
 #if defined(CS_DEVSTATE_FEATURE) && defined(CS_AST_HAS_EVENT)
 	sccp_devstate_specifier_t *devstateSpecifier;
 #endif
 	char family[25];
 
-	if ((d = sccp_device_retain(d))) {
+	if (d) {
 		sccp_log((DEBUGCAT_CORE + DEBUGCAT_DEVICE)) (VERBOSE_PREFIX_1 "SCCP: Clean Device %s\n", d->id);
 		sccp_dev_set_registered(d, SKINNY_DEVICE_RS_NONE);						/* set correct register state */
 		if (remove_from_global) {
@@ -1859,25 +1876,23 @@ void sccp_dev_clean(sccp_device_t * d, boolean_t remove_from_global, uint8_t cle
 		sccp_config_cleanup_dynamically_allocated_memory(d, SCCP_CONFIG_DEVICE_SEGMENT);
 
 		/* hang up open channels and remove device from line */
-		sccp_device_t *tmpDevice = NULL;
 
 		SCCP_LIST_LOCK(&d->buttonconfig);
 		SCCP_LIST_TRAVERSE(&d->buttonconfig, config, list) {
 			if (config->type == LINE) {
-				line = sccp_line_find_byname(config->button.line.name, FALSE);
+				AUTO_RELEASE sccp_line_t *line = sccp_line_find_byname(config->button.line.name, FALSE);
 				if (!line) {
 					continue;
 				}
 				SCCP_LIST_LOCK(&line->channels);
-				SCCP_LIST_TRAVERSE_SAFE_BEGIN(&line->channels, channel, list) {
-					if (sccp_channel_retain(channel)) {
-						tmpDevice = sccp_channel_getDevice_retained(channel);
+				SCCP_LIST_TRAVERSE_SAFE_BEGIN(&line->channels, c, list) {
+					AUTO_RELEASE sccp_channel_t *channel = sccp_channel_retain(c);
+					if (c) {
+						AUTO_RELEASE sccp_device_t *tmpDevice = sccp_channel_getDevice_retained(channel);
 						if (tmpDevice == d) {
 							sccp_log((DEBUGCAT_CORE + DEBUGCAT_DEVICE)) (VERBOSE_PREFIX_2 "SCCP: Hangup open channel on line %s device %s\n", line->name, d->id);
 							sccp_channel_endcall(channel);
 						}
-						tmpDevice = tmpDevice ? sccp_device_release(tmpDevice) : NULL;
-						sccp_channel_release(channel);
 					}
 				}
 				SCCP_LIST_TRAVERSE_SAFE_END;
@@ -1886,7 +1901,6 @@ void sccp_dev_clean(sccp_device_t * d, boolean_t remove_from_global, uint8_t cle
 				/* remove devices from line */
 				sccp_log((DEBUGCAT_CORE + DEBUGCAT_DEVICE)) (VERBOSE_PREFIX_2 "SCCP: Remove Line %s from device %s\n", line->name, d->id);
 				sccp_line_removeDevice(line, d);
-				line = sccp_line_release(line);
 			}
 			config->instance = 0;									/* reset button configuration to rebuild template on register */
 		}
@@ -1956,8 +1970,6 @@ void sccp_dev_clean(sccp_device_t * d, boolean_t remove_from_global, uint8_t cle
 		}
 		SCCP_LIST_UNLOCK(&d->devstateSpecifiers);
 #endif
-
-		d = sccp_device_release(d);
 	}
 }
 
@@ -2196,7 +2208,7 @@ void sccp_device_sendcallstate(const sccp_device_t * d, uint8_t instance, uint32
 	msg->data.CallStateMessage.lel_priority = htolel(priority);
 	/*r->data.CallStateMessage.lel_unknown3 = htolel(2); */
 	sccp_dev_send(d, msg);
-	sccp_log((DEBUGCAT_DEVICE)) (VERBOSE_PREFIX_3 "%s: Send and Set the call state %s(%d) on call %d\n", d->id, sccp_callstate2str(state), state, callid);
+	sccp_log((DEBUGCAT_DEVICE)) (VERBOSE_PREFIX_3 "%s: Send and Set the call state %s(%d) on call %d\n", d->id, skinny_callstate2str(state), state, callid);
 }
 
 /*!
@@ -2211,7 +2223,6 @@ uint8_t sccp_device_numberOfChannels(const sccp_device_t * device)
 {
 	sccp_buttonconfig_t *config;
 	sccp_channel_t *c;
-	sccp_line_t *l;
 	uint8_t numberOfChannels = 0;
 
 	if (!device) {
@@ -2219,24 +2230,20 @@ uint8_t sccp_device_numberOfChannels(const sccp_device_t * device)
 		return 0;
 	}
 
-	sccp_device_t *tmpDevice = NULL;
-
 	SCCP_LIST_TRAVERSE(&device->buttonconfig, config, list) {
 		if (config->type == LINE) {
-			l = sccp_line_find_byname(config->button.line.name, FALSE);
+			AUTO_RELEASE sccp_line_t *l = sccp_line_find_byname(config->button.line.name, FALSE);
 			if (!l) {
 				continue;
 			}
 			SCCP_LIST_LOCK(&l->channels);
 			SCCP_LIST_TRAVERSE(&l->channels, c, list) {
-				tmpDevice = sccp_channel_getDevice_retained(c);
+				AUTO_RELEASE sccp_device_t *tmpDevice = sccp_channel_getDevice_retained(c);
 				if (tmpDevice == device) {
 					numberOfChannels++;
 				}
-				tmpDevice = tmpDevice ? sccp_device_release(tmpDevice) : NULL;
 			}
 			SCCP_LIST_UNLOCK(&l->channels);
-			l = sccp_line_release(l);
 		}
 	}
 
@@ -2288,7 +2295,7 @@ static void sccp_device_old_indicate_remoteHold(const sccp_device_t * device, ui
 {
 	sccp_device_sendcallstate(device, lineInstance, callid, SKINNY_CALLSTATE_HOLD, callpriority, callPrivacy);
 	sccp_dev_set_keyset(device, lineInstance, callid, KEYMODE_ONHOLD);
-	sccp_dev_displayprompt(device, lineInstance, callid, SKINNY_DISP_HOLD, 0);
+	sccp_dev_displayprompt(device, lineInstance, callid, SKINNY_DISP_HOLD, GLOB(digittimeout));
 }
 
 /*!
@@ -2298,7 +2305,7 @@ static void sccp_device_new_indicate_remoteHold(const sccp_device_t * device, ui
 {
 	sccp_device_sendcallstate(device, lineInstance, callid, SKINNY_CALLSTATE_HOLDRED, callpriority, callPrivacy);
 	sccp_dev_set_keyset(device, lineInstance, callid, KEYMODE_ONHOLD);
-	sccp_dev_displayprompt(device, lineInstance, callid, SKINNY_DISP_HOLD, 0);
+	sccp_dev_displayprompt(device, lineInstance, callid, SKINNY_DISP_HOLD, GLOB(digittimeout));
 }
 
 static void sccp_device_indicate_offhook(const sccp_device_t * device, sccp_linedevices_t * linedevice, uint8_t callid)
@@ -2307,25 +2314,36 @@ static void sccp_device_indicate_offhook(const sccp_device_t * device, sccp_line
 	sccp_dev_set_speaker(device, SKINNY_STATIONSPEAKER_ON);
 	sccp_device_sendcallstate(device, linedevice->lineInstance, callid, SKINNY_CALLSTATE_OFFHOOK, SKINNY_CALLPRIORITY_LOW, SKINNY_CALLINFO_VISIBILITY_DEFAULT);
 	sccp_dev_set_cplane(device, linedevice->lineInstance, 1);
-	sccp_dev_displayprompt(device, linedevice->lineInstance, callid, SKINNY_DISP_ENTER_NUMBER, 0);
+	sccp_dev_displayprompt(device, linedevice->lineInstance, callid, SKINNY_DISP_ENTER_NUMBER, GLOB(digittimeout));
 	sccp_dev_set_keyset(device, linedevice->lineInstance, callid, KEYMODE_OFFHOOK);
 	sccp_dev_starttone(device, SKINNY_TONE_INSIDEDIALTONE, linedevice->lineInstance, callid, 0);
 }
 
-static void sccp_device_indicate_onhook(const sccp_device_t * device, const uint8_t lineInstance, uint8_t callid)
+static void sccp_device_indicate_onhook(const sccp_device_t * device, const sccp_channel_t *channel, const uint8_t lineInstance, uint8_t callid)
 {
 	sccp_dev_stoptone(device, lineInstance, callid);
+	sccp_device_setLamp(device, SKINNY_STIMULUS_LINE, lineInstance, SKINNY_LAMP_OFF);
+
+/*new*/	// sccp_dev_cleardisplayprinotify(device);					/* not sure */
 	sccp_dev_cleardisplaynotify(device);
 	sccp_dev_clearprompt(device, lineInstance, callid);
+/*new*/	// sccp_dev_clearprompt(device, 0, 0);						/* already performed by sccp_dev_check_displayprompt */
+/*new*/	sccp_dev_cleardisplay(device);
 
-	//sccp_device_sendcallstate(d, instance, c->callid, SKINNY_CALLSTATE_CONNECTED, SKINNY_CALLPRIORITY_LOW, SKINNY_CALLINFO_VISIBILITY_HIDDEN);	/** if channel was answered somewhere, set state to connected before onhook -> no missedCalls entry */
+	if (channel && channel->answered_elsewhere) {
+		sccp_device_sendcallstate(device, lineInstance, channel->callid, SKINNY_CALLSTATE_CONNECTED, SKINNY_CALLPRIORITY_LOW, SKINNY_CALLINFO_VISIBILITY_HIDDEN);
+	}
 
 	sccp_dev_set_ringer(device, SKINNY_RINGTYPE_OFF, lineInstance, callid);
 	sccp_device_sendcallstate(device, lineInstance, callid, SKINNY_CALLSTATE_ONHOOK, SKINNY_CALLPRIORITY_LOW, SKINNY_CALLINFO_VISIBILITY_DEFAULT);
 	sccp_dev_set_cplane(device, lineInstance, 0);
 	sccp_dev_set_keyset(device, lineInstance, callid, KEYMODE_ONHOOK);
 	sccp_handle_time_date_req(device->session, (sccp_device_t *)device, NULL);	/** we need datetime on hangup for 7936 */
-	sccp_dev_set_speaker(device, SKINNY_STATIONSPEAKER_OFF);
+	
+	if (channel && channel == device->active_channel) {
+		sccp_dev_set_speaker(device, SKINNY_STATIONSPEAKER_OFF);
+	}
+	sccp_dev_check_displayprompt(device);						/* see if we need to display anything from the messageStack */
 }
 
 static void sccp_device_indicate_offhook_remote(const sccp_device_t * device, sccp_linedevices_t * linedevice, const sccp_channel_t * channel){
@@ -2355,9 +2373,8 @@ static void sccp_device_indicate_connected(const sccp_device_t * device, sccp_li
 		device->protocol->sendCallInfo(device, channel, linedevice->lineInstance);
 	}
 	sccp_dev_set_cplane(device, linedevice->lineInstance, 1);
-	sccp_dev_set_keyset(device, linedevice->lineInstance, channel->callid, KEYMODE_CONNECTED);
-	sccp_dev_displayprompt(device, linedevice->lineInstance, channel->callid, SKINNY_DISP_CONNECTED, 0);
-
+//	sccp_dev_set_keyset(device, linedevice->lineInstance, channel->callid, KEYMODE_CONNECTED);		// moved back to sccp_indicate (re-used for connconf as well)
+	sccp_dev_displayprompt(device, linedevice->lineInstance, channel->callid, SKINNY_DISP_CONNECTED, GLOB(digittimeout));
 }
 
 static void sccp_device_indicate_dialing(const sccp_device_t * device, const uint8_t lineInstance, const sccp_channel_t * channel){
@@ -2380,14 +2397,14 @@ static void sccp_device_indicate_proceed(const sccp_device_t * device, const uin
 	if (device->protocol && device->protocol->sendCallInfo) {
 		device->protocol->sendCallInfo(device, channel, lineInstance);
 	}
-	sccp_dev_displayprompt(device, lineInstance, channel->callid, SKINNY_DISP_CALL_PROCEED, 0);
+	sccp_dev_displayprompt(device, lineInstance, channel->callid, SKINNY_DISP_CALL_PROCEED, GLOB(digittimeout));
 }
 
 
 /*!
  * \brief Add message to the MessageStack to be shown on the Status Line of the SCCP Device
  */
-void sccp_device_addMessageToStack(sccp_device_t * device, const uint8_t priority, const char *message)
+void sccp_device_addMessageToStack(const sccp_device_t * device, const uint8_t priority, const char *message)
 {
 	// sccp_log((DEBUGCAT_CORE + DEBUGCAT_DEVICE + DEBUGCAT_MESSAGE)) (VERBOSE_PREFIX_1 "%s: (sccp_device_addMessageToStack), '%s' at priority %d \n", DEV_ID_LOG(device), message, priority);
 	if (ARRAY_LEN(device->messageStack) <= priority) {
@@ -2411,7 +2428,7 @@ void sccp_device_addMessageToStack(sccp_device_t * device, const uint8_t priorit
 /*!
  * \brief Remove a message from the MessageStack to be shown on the Status Line of the SCCP Device
  */
-void sccp_device_clearMessageFromStack(sccp_device_t * device, const uint8_t priority)
+void sccp_device_clearMessageFromStack(const sccp_device_t * device, const uint8_t priority)
 {
 	if (ARRAY_LEN(device->messageStack) <= priority) {
 		return;
@@ -2536,10 +2553,14 @@ void sccp_device_featureChangedDisplay(const sccp_event_t * event)
  */
 static sccp_push_result_t sccp_device_pushURL(const sccp_device_t * device, const char *url, uint8_t priority, uint8_t tone)
 {
-	char xmlData[512];
-
-	sprintf(xmlData, "<CiscoIPPhoneExecute><ExecuteItem Priority=\"0\"URL=\"%s\"/></CiscoIPPhoneExecute>", url);
-
+	const char *xmlFormat = "<CiscoIPPhoneExecute><ExecuteItem Priority=\"0\" URL=\"%s\"/></CiscoIPPhoneExecute>";
+	size_t msg_length = strlen(xmlFormat) + sccp_strlen(url) - 2 /* for %s */ + 1 /* for terminator */;
+	if (sccp_strlen(url) > 256) {
+		sccp_log((DEBUGCAT_DEVICE))(VERBOSE_PREFIX_3 "%s: (pushURL) url is to long (max 256 char).\n", DEV_ID_LOG(device));
+		return SCCP_PUSH_RESULT_FAIL;
+	}
+	char xmlData[msg_length];
+	snprintf(xmlData, msg_length, xmlFormat, url);
 	device->protocol->sendUserToDeviceDataVersionMessage(device, 0, 0, 1, 1, xmlData, priority);
 	if (SKINNY_TONE_SILENCE != tone) {
 		sccp_dev_starttone(device, tone, 0, 0, 0);
@@ -2549,13 +2570,37 @@ static sccp_push_result_t sccp_device_pushURL(const sccp_device_t * device, cons
 
 /*!
  * \brief Push a Text Message to an SCCP device
- */
+ *
+ * \note
+ * title field can be max 32 characters long
+ * protocolversion < 17 allows for maximum of 1024 characters in the text block / maximum 2000 characted in overall message 
+ * protocolversion > 17 allows variable sized messages up to 4000 char in the text block (using multiple messages if necessary) 
+ */ 
 static sccp_push_result_t sccp_device_pushTextMessage(const sccp_device_t * device, const char *messageText, const char *from, uint8_t priority, uint8_t tone)
 {
-	char xmlData[1024];
+	const char *xmlFormat = "<CiscoIPPhoneText>%s<Text>%s</Text></CiscoIPPhoneText>";
+	size_t msg_length = strlen(xmlFormat) + sccp_strlen(messageText) - 4 /* for the %s' */ + 1 /* for terminator */;
 
-	sprintf(xmlData, "<CiscoIPPhoneText><Title>%s</Title><Text>%s</Text></CiscoIPPhoneText>", from ? from : "", messageText);
+	if (sccp_strlen(from) > 32) {
+		sccp_log((DEBUGCAT_DEVICE))(VERBOSE_PREFIX_3 "%s: (pushTextMessage) from is to long (max 32 char).\n", DEV_ID_LOG(device));
+		return SCCP_PUSH_RESULT_FAIL;
+	}
 
+	if ((device->protocolversion < 17 && 1024 > msg_length) || sccp_strlen(messageText) > 4000) {
+		sccp_log((DEBUGCAT_DEVICE))(VERBOSE_PREFIX_3 "%s: (pushTextMessage) messageText is to long.\n", DEV_ID_LOG(device));
+		return SCCP_PUSH_RESULT_FAIL;
+	}
+
+	const char *xmlTitleFormat = "<Title>%s</Title>";
+	size_t title_length = strlen(xmlTitleFormat) +sccp_strlen(from)- 2 /* for the %s */ + 1 /* for terminator */;
+	char title[title_length];
+	if (!sccp_strlen_zero(from)) {
+		msg_length += title_length;
+		snprintf(title, title_length, xmlTitleFormat, from);
+	}
+	
+	char xmlData[msg_length];
+	snprintf(xmlData, msg_length, xmlFormat, title,  messageText);
 	device->protocol->sendUserToDeviceDataVersionMessage(device, 0, 0, 1, 1, xmlData, priority);
 
 	if (SKINNY_TONE_SILENCE != tone) {
