@@ -10,8 +10,8 @@
  * \remarks	Purpose:	SCCP Hint
  * 		When to use:	Does the business of hint status
  *
- * $Date: 2011-01-04 17:29:12 +0100 (Tue, 04 Jan 2011) $
- * $Revision: 2215 $
+ * $Date: 2014-06-10 12:28:53 +0200 (Tue, 10 Jun 2014) $
+ * $Revision: 5498 $
  */
 
 /*!
@@ -34,7 +34,7 @@
 #include "sccp_line.h"
 #include "sccp_utils.h"
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 2215 $")
+SCCP_FILE_VERSION(__FILE__, "$Revision: 5498 $")
 
 /* ========================================================================================================================= Struct Definitions */
 /*!
@@ -189,7 +189,6 @@ void sccp_hint_module_stop(void)
 
 	{
 		sccp_hint_list_t *hint;
-		sccp_device_t *device;
 		sccp_hint_SubscribingDevice_t *subscriber;
 
 		SCCP_LIST_LOCK(&sccp_hint_subscriptions);
@@ -202,9 +201,9 @@ void sccp_hint_module_stop(void)
 			// All subscriptions that have this device should be removed, force cleanup 
 			SCCP_LIST_LOCK(&hint->subscribers);
 			while ((subscriber = SCCP_LIST_REMOVE_HEAD(&hint->subscribers, list))) {
-				if ((device = sccp_device_retain((sccp_device_t *) subscriber->device))) {
+				AUTO_RELEASE sccp_device_t *device = sccp_device_retain((sccp_device_t *) subscriber->device);
+				if (device) {
 					subscriber->device = sccp_device_release(subscriber->device);
-					device = sccp_device_release(device);
 					sccp_free(subscriber);
 				}
 			}
@@ -380,9 +379,9 @@ static void sccp_hint_deviceRegistered(const sccp_device_t * device)
 {
 	sccp_buttonconfig_t *config;
 	uint8_t positionOnDevice = 0;
-	sccp_device_t *d = NULL;
-
-	if ((d = sccp_device_retain((sccp_device_t *) device))) {
+	
+	AUTO_RELEASE sccp_device_t *d = sccp_device_retain((sccp_device_t *) device);
+	if (d) {
 		SCCP_LIST_TRAVERSE(&device->buttonconfig, config, list) {
 			positionOnDevice++;
 
@@ -393,7 +392,6 @@ static void sccp_hint_deviceRegistered(const sccp_device_t * device)
 				sccp_hint_addSubscription4Device(device, config->button.speeddial.hint, config->instance, positionOnDevice);
 			}
 		}
-		sccp_device_release(d);
 	}
 }
 
@@ -462,6 +460,14 @@ static void sccp_hint_addSubscription4Device(const sccp_device_t * device, const
 		hint_context = GLOB(context);
 	}
 
+	/* skip subscribtions to already owned line */
+/*	sccp_buttonconfig_t *config;
+	SCCP_LIST_TRAVERSE(&device->buttonconfig, config, list) {
+		if (config->type == LINE && sccp_strcaseequals(config->button.line.name, hint_exten)) {
+			sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_3 "%s: (sccp_hint_addSubscription4Device) Skipping Hint Registration for Line we already have this connected (%s, %s).\n", DEV_ID_LOG(device), config->button.line.name, hint_exten);
+			return;
+		}
+	}*/
 	sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_3 "%s: (sccp_hint_addSubscription4Device) Dialplan %s for exten: %s and context: %s\n", DEV_ID_LOG(device), hintStr, hint_exten, hint_context);
 
 	SCCP_LIST_TRAVERSE(&sccp_hint_subscriptions, hint, list) {
@@ -616,10 +622,9 @@ static void sccp_hint_lineStatusChanged(sccp_line_t * line, sccp_device_t * devi
  */
 static void sccp_hint_updateLineState(struct sccp_hint_lineState *lineState)
 {
-	sccp_line_t *line = NULL;
-
-	if ((line = sccp_line_retain(lineState->line))) {
-		sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_4 "%s: (sccp_hint_updateLineState) Update Line Channel State: %s(%d)\n", line->name, channelstate2str(lineState->state), lineState->state);
+	AUTO_RELEASE sccp_line_t *line = sccp_line_retain(lineState->line);
+	if (line) {
+		sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_4 "%s: (sccp_hint_updateLineState) Update Line Channel State: %s(%d)\n", line->name, sccp_channelstate2str(lineState->state), lineState->state);
 
 		/* no line, or line without devices */
 		if (0 == SCCP_LIST_GETSIZE(&line->devices)) {
@@ -639,8 +644,6 @@ static void sccp_hint_updateLineState(struct sccp_hint_lineState *lineState)
 
 		/* push chagnes to pbx */
 		sccp_hint_notifyPBX(lineState);
-
-		line = sccp_line_release(line);
 	}
 }
 
@@ -651,7 +654,6 @@ static void sccp_hint_updateLineState(struct sccp_hint_lineState *lineState)
 static void sccp_hint_updateLineStateForSharedLine(struct sccp_hint_lineState *lineState)
 {
 	sccp_line_t *line = lineState->line;
-	sccp_channel_t *channel = NULL;
 
 	memset(lineState->callInfo.partyName, 0, sizeof(lineState->callInfo.partyName));
 	memset(lineState->callInfo.partyNumber, 0, sizeof(lineState->callInfo.partyNumber));
@@ -663,9 +665,10 @@ static void sccp_hint_updateLineStateForSharedLine(struct sccp_hint_lineState *l
 		sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_4 "%s: (sccp_hint_updateLineStateForSharedLine) number of active channels %d\n", line->name, SCCP_LIST_GETSIZE(&line->channels));
 		if (SCCP_LIST_GETSIZE(&line->channels) == 1) {
 			SCCP_LIST_LOCK(&line->channels);
-			channel = SCCP_LIST_FIRST(&line->channels);
+			AUTO_RELEASE sccp_channel_t *channel = SCCP_LIST_FIRST(&line->channels) ? sccp_channel_retain(SCCP_LIST_FIRST(&line->channels)) : NULL;
 			SCCP_LIST_UNLOCK(&line->channels);
-			if (channel && (channel = sccp_channel_retain(channel))) {
+
+			if (channel) {
 				lineState->callInfo.calltype = channel->calltype;
 
 				if (channel->state != SCCP_CHANNELSTATE_ONHOOK && channel->state != SCCP_CHANNELSTATE_DOWN) {
@@ -673,16 +676,25 @@ static void sccp_hint_updateLineStateForSharedLine(struct sccp_hint_lineState *l
 
 					/* set cid name/numbe information according to the call direction */
 					if (SKINNY_CALLTYPE_INBOUND == channel->calltype) {
-						sccp_copy_string(lineState->callInfo.partyName, channel->callInfo.callingPartyName, sizeof(lineState->callInfo.partyName));
-						sccp_copy_string(lineState->callInfo.partyNumber, channel->callInfo.callingPartyName, sizeof(lineState->callInfo.partyNumber));
+						if (channel->privacy || !channel->callInfo.presentation) {
+							sccp_copy_string(lineState->callInfo.partyName, SKINNY_DISP_PRIVATE, sizeof(lineState->callInfo.partyName));
+							sccp_copy_string(lineState->callInfo.partyNumber, SKINNY_DISP_PRIVATE, sizeof(lineState->callInfo.partyNumber));
+						} else {
+							sccp_copy_string(lineState->callInfo.partyName, channel->callInfo.callingPartyName, sizeof(lineState->callInfo.partyName));
+							sccp_copy_string(lineState->callInfo.partyNumber, channel->callInfo.callingPartyName, sizeof(lineState->callInfo.partyNumber));
+						}
 					} else {
-						sccp_copy_string(lineState->callInfo.partyName, channel->callInfo.calledPartyName, sizeof(lineState->callInfo.partyName));
-						sccp_copy_string(lineState->callInfo.partyNumber, channel->callInfo.calledPartyNumber, sizeof(lineState->callInfo.partyNumber));
+						if (channel->privacy || !channel->callInfo.presentation) {
+							sccp_copy_string(lineState->callInfo.partyName, SKINNY_DISP_PRIVATE, sizeof(lineState->callInfo.partyName));
+							sccp_copy_string(lineState->callInfo.partyNumber, SKINNY_DISP_PRIVATE, sizeof(lineState->callInfo.partyNumber));
+						} else {
+							sccp_copy_string(lineState->callInfo.partyName, channel->callInfo.calledPartyName, sizeof(lineState->callInfo.partyName));
+							sccp_copy_string(lineState->callInfo.partyNumber, channel->callInfo.calledPartyNumber, sizeof(lineState->callInfo.partyNumber));
+						}
 					}
 				} else {
 					lineState->state = SCCP_CHANNELSTATE_ONHOOK;
 				}
-				channel = sccp_channel_release(channel);
 			} else {
 				lineState->state = SCCP_CHANNELSTATE_ONHOOK;
 			}
@@ -698,7 +710,7 @@ static void sccp_hint_updateLineStateForSharedLine(struct sccp_hint_lineState *l
 		lineState->state = SCCP_CHANNELSTATE_ONHOOK;
 	}
 
-	sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_4 "%s: (sccp_hint_updateLineStateForSharedLine) Set sharedLineState to %s(%d)\n", line->name, channelstate2str(lineState->state), lineState->state);
+	sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_4 "%s: (sccp_hint_updateLineStateForSharedLine) Set sharedLineState to %s(%d)\n", line->name, sccp_channelstate2str(lineState->state), lineState->state);
 }
 
 /*!
@@ -709,9 +721,6 @@ static void sccp_hint_updateLineStateForSharedLine(struct sccp_hint_lineState *l
 static void sccp_hint_updateLineStateForSingleLine(struct sccp_hint_lineState *lineState)
 {
 	sccp_line_t *line = lineState->line;
-	sccp_channel_t *channel = NULL;
-	sccp_device_t *device = NULL;
-	sccp_linedevices_t *lineDevice = NULL;
 	uint8_t state;
 
 	//boolean_t dev_privacy = FALSE;
@@ -721,27 +730,25 @@ static void sccp_hint_updateLineStateForSingleLine(struct sccp_hint_lineState *l
 	memset(lineState->callInfo.partyNumber, 0, sizeof(lineState->callInfo.partyNumber));
 
 	SCCP_LIST_LOCK(&line->channels);
-	channel = SCCP_LIST_FIRST(&line->channels);
+	AUTO_RELEASE sccp_channel_t *channel = SCCP_LIST_FIRST(&line->channels) ? sccp_channel_retain(SCCP_LIST_FIRST(&line->channels)) : NULL;
 	SCCP_LIST_UNLOCK(&line->channels);
-	if (channel && (channel = sccp_channel_retain(channel))) {
+
+	if (channel) {
 		lineState->callInfo.calltype = channel->calltype;
 		state = channel->state;
 
 		SCCP_LIST_LOCK(&line->devices);
-		lineDevice = SCCP_LIST_FIRST(&line->devices);
+		AUTO_RELEASE sccp_linedevices_t *lineDevice = SCCP_LIST_FIRST(&line->devices) ? sccp_linedevice_retain(SCCP_LIST_FIRST(&line->devices)) : NULL;
 		SCCP_LIST_UNLOCK(&line->devices);
-
-		if ((lineDevice = sccp_linedevice_retain(lineDevice))) {
-			if ((device = sccp_device_retain(lineDevice->device))) {
+		if (lineDevice) {
+			AUTO_RELEASE sccp_device_t *device = sccp_device_retain(lineDevice->device);
+			if (device) {
 				if (device->dndFeature.enabled && device->dndFeature.status == SCCP_DNDMODE_REJECT) {
 					state = SCCP_CHANNELSTATE_DND;
 				}
 				//dev_privacy = device->privacyFeature.enabled;
-				device = device ? sccp_device_release(device) : NULL;
 			}
-			lineDevice = lineDevice ? sccp_linedevice_release(lineDevice) : NULL;
 		}
-
 		switch (state) {
 			case SCCP_CHANNELSTATE_DOWN:
 				state = SCCP_CHANNELSTATE_ONHOOK;
@@ -776,13 +783,23 @@ static void sccp_hint_updateLineStateForSingleLine(struct sccp_hint_lineState *l
 				/** set cid name/number information according to the call direction */
 				switch (channel->calltype) {
 					case SKINNY_CALLTYPE_INBOUND:
-						sccp_copy_string(lineState->callInfo.partyName, channel->callInfo.callingPartyName, sizeof(lineState->callInfo.partyName));
-						sccp_copy_string(lineState->callInfo.partyNumber, channel->callInfo.callingPartyNumber, sizeof(lineState->callInfo.partyNumber));
+						if (channel->privacy || !channel->callInfo.presentation) {
+							sccp_copy_string(lineState->callInfo.partyName, SKINNY_DISP_PRIVATE, sizeof(lineState->callInfo.partyName));
+							sccp_copy_string(lineState->callInfo.partyNumber, SKINNY_DISP_PRIVATE, sizeof(lineState->callInfo.partyNumber));
+						} else {
+							sccp_copy_string(lineState->callInfo.partyName, channel->callInfo.callingPartyName, sizeof(lineState->callInfo.partyName));
+							sccp_copy_string(lineState->callInfo.partyNumber, channel->callInfo.callingPartyNumber, sizeof(lineState->callInfo.partyNumber));
+						}
 						sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_4 "%s: set speeddial partyName: '%s' (callingParty)\n", line->name, channel->callInfo.callingPartyName);
 						break;
 					case SKINNY_CALLTYPE_OUTBOUND:
-						sccp_copy_string(lineState->callInfo.partyName, channel->callInfo.calledPartyName, sizeof(lineState->callInfo.partyName));
-						sccp_copy_string(lineState->callInfo.partyNumber, channel->callInfo.calledPartyNumber, sizeof(lineState->callInfo.partyNumber));
+						if (channel->privacy || !channel->callInfo.presentation) {
+							sccp_copy_string(lineState->callInfo.partyName, SKINNY_DISP_PRIVATE, sizeof(lineState->callInfo.partyName));
+							sccp_copy_string(lineState->callInfo.partyNumber, SKINNY_DISP_PRIVATE, sizeof(lineState->callInfo.partyNumber));
+						} else {
+							sccp_copy_string(lineState->callInfo.partyName, channel->callInfo.calledPartyName, sizeof(lineState->callInfo.partyName));
+							sccp_copy_string(lineState->callInfo.partyNumber, channel->callInfo.calledPartyNumber, sizeof(lineState->callInfo.partyNumber));
+						}
 						sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_4 "%s: set speeddial partyName: '%s' (calledParty)\n", line->name, channel->callInfo.calledPartyName);
 						break;
 					case SKINNY_CALLTYPE_FORWARD:
@@ -790,6 +807,8 @@ static void sccp_hint_updateLineStateForSingleLine(struct sccp_hint_lineState *l
 						sccp_copy_string(lineState->callInfo.partyNumber, "cfwd", sizeof(lineState->callInfo.partyNumber));
 						sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_4 "%s: set speedial partyName: cfwd\n", line->name);
 						break;
+					case skinny_calltype_LOOKUPERROR:
+						break;
 				}
 				break;
 		}
@@ -797,14 +816,13 @@ static void sccp_hint_updateLineStateForSingleLine(struct sccp_hint_lineState *l
 		sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_4 "%s: (sccp_hint_updateLineStateForSingleLine) partyName: %s, partyNumber: %s\n", line->name, lineState->callInfo.partyName, lineState->callInfo.partyNumber);
 
 		lineState->state = state;
-		channel = sccp_channel_release(channel);
 	} else {
 		sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_4 "%s: (sccp_hint_updateLineStateForSingleLine) NO CHANNEL\n", line->name);
 		lineState->state = SCCP_CHANNELSTATE_ONHOOK;
 		sccp_hint_checkForDND(lineState);
 	}													// if(channel)
 
-	sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_4 "%s: (sccp_hint_updateLineStateForSingleLine) Set singleLineState to %s(%d)\n", line->name, channelstate2str(lineState->state), lineState->state);
+	sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_4 "%s: (sccp_hint_updateLineStateForSingleLine) Set singleLineState to %s(%d)\n", line->name, sccp_channelstate2str(lineState->state), lineState->state);
 }
 
 /* ========================================================================================================================= Event Handlers : Feature Change */
@@ -818,27 +836,26 @@ static void sccp_hint_updateLineStateForSingleLine(struct sccp_hint_lineState *l
 static void sccp_hint_handleFeatureChangeEvent(const sccp_event_t * event)
 {
 	sccp_buttonconfig_t *buttonconfig = NULL;
-	sccp_device_t *d = NULL;
-	sccp_line_t *line = NULL;
 
 	switch (event->event.featureChanged.featureType) {
 		case SCCP_FEATURE_DND:
-			if ((d = sccp_device_retain(event->event.featureChanged.device))) {
+		{
+			AUTO_RELEASE sccp_device_t *d = sccp_device_retain(event->event.featureChanged.device);
+			if (d) {
 				SCCP_LIST_LOCK(&d->buttonconfig);
 				SCCP_LIST_TRAVERSE(&d->buttonconfig, buttonconfig, list) {
 					if (buttonconfig->type == LINE) {
-						line = sccp_line_find_byname(buttonconfig->button.line.name, FALSE);
+						AUTO_RELEASE sccp_line_t *line = sccp_line_find_byname(buttonconfig->button.line.name, FALSE);
 						if (line) {
 							sccp_log((DEBUGCAT_SOFTKEY)) (VERBOSE_PREFIX_3 "%s: (sccp_hint_handleFeatureChangeEvent) Notify the dnd status (%s) to asterisk for line %s\n", d->id, d->dndFeature.status ? "on" : "off", line->name);
 							sccp_hint_lineStatusChanged(line, d);
-							line = sccp_line_release(line);
 						}
 					}
 				}
 				SCCP_LIST_UNLOCK(&d->buttonconfig);
-				sccp_device_release(d);
 			}
 			break;
+		}
 		default:
 			break;
 	}
@@ -889,6 +906,8 @@ static enum ast_device_state sccp_hint_hint2DeviceState(sccp_channelstate_t stat
 		case SCCP_CHANNELSTATE_CALLREMOTEMULTILINE:
 			newDeviceState = AST_DEVICE_INUSE;
 			break;
+		case sccp_channelstate_LOOKUPERROR:
+			break;
 	}
 	return newDeviceState;
 }
@@ -921,7 +940,7 @@ static void sccp_hint_notifyPBX(struct sccp_hint_lineState *lineState)
 	}
 #endif
 
-	sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_3 "SCCP: (sccp_hint_notifyPBX) Notify asterisk to set state to sccp channelstate %s (%d) => asterisk: %s (%d) on channel SCCP/%s\n", channelstate2str(lineState->state), lineState->state, pbxdevicestate2str(newDeviceState), newDeviceState, lineState->line->name);
+	sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_3 "SCCP: (sccp_hint_notifyPBX) Notify asterisk to set state to sccp channelstate %s (%d) => asterisk: %s (%d) on channel SCCP/%s\n", sccp_channelstate2str(lineState->state), lineState->state, pbxsccp_devicestate2str(newDeviceState), newDeviceState, lineState->line->name);
 
 	// if pbx devicestate does not change, no need to inform asterisk */
 	// if (hint && lineState->state == hint->currentState) {
@@ -934,7 +953,7 @@ static void sccp_hint_notifyPBX(struct sccp_hint_lineState *lineState)
 		event = pbx_event_new(AST_EVENT_DEVICE_STATE, AST_EVENT_IE_DEVICE, AST_EVENT_IE_PLTYPE_STR, channelName, AST_EVENT_IE_STATE, AST_EVENT_IE_PLTYPE_UINT, newDeviceState, AST_EVENT_IE_CEL_CIDNAME, AST_EVENT_IE_PLTYPE_STR, lineState->callInfo.partyName, AST_EVENT_IE_CEL_CIDNUM, AST_EVENT_IE_PLTYPE_STR, lineState->callInfo.partyNumber, AST_EVENT_IE_CEL_USERFIELD, AST_EVENT_IE_PLTYPE_UINT, lineState->callInfo.calltype, AST_EVENT_IE_END);
 		pbx_event_queue_and_cache(event);
 		sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_4 "SCCP: \n");
-		sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_3 "SCCP: (sccp_hint_notifyPBX!distributed) Notify asterisk to set state to sccp channelstate %s (%d) => asterisk: %s (%d) on channel SCCP/%s\n", channelstate2str(lineState->state), lineState->state, pbxdevicestate2str(newDeviceState), newDeviceState, lineState->line->name);
+		sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_3 "SCCP: (sccp_hint_notifyPBX!distributed) Notify asterisk to set state to sccp channelstate %s (%d) => asterisk: %s (%d) on channel SCCP/%s\n", sccp_channelstate2str(lineState->state), lineState->state, pbxsccp_devicestate2str(newDeviceState), newDeviceState, lineState->line->name);
 		pbx_devstate_changed_literal(newDeviceState, channelName);					/* come back via pbx callback and update subscribers */
 #else
 		pbx_devstate_changed_literal(newDeviceState, channelName);					/* come back via pbx callback and update subscribers */
@@ -952,7 +971,6 @@ static void sccp_hint_notifyPBX(struct sccp_hint_lineState *lineState)
  */
 static void sccp_hint_notifySubscribers(sccp_hint_list_t * hint)
 {
-	sccp_device_t *d;
 	sccp_hint_SubscribingDevice_t *subscriber = NULL;
 	sccp_msg_t *msg;
 
@@ -973,7 +991,7 @@ static void sccp_hint_notifySubscribers(sccp_hint_list_t * hint)
 		return;
 	}
 
-	sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_3 "%s: (sccp_hint_notifySubscribers) notify %u subscriber(s) of %s's state %s\n", hint->exten, SCCP_LIST_GETSIZE(&hint->subscribers), (hint->hint_dialplan) ? hint->hint_dialplan : "null", channelstate2str(hint->currentState));
+	sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_3 "%s: (sccp_hint_notifySubscribers) notify %u subscriber(s) of %s's state %s\n", hint->exten, SCCP_LIST_GETSIZE(&hint->subscribers), (hint->hint_dialplan) ? hint->hint_dialplan : "null", sccp_channelstate2str(hint->currentState));
 
 	/* use a temporary channel as fallback for non dynamic speeddial devices */
 	memset(&tmpChannel, 0, sizeof(sccp_channel_t));
@@ -986,9 +1004,9 @@ static void sccp_hint_notifySubscribers(sccp_hint_list_t * hint)
 
 	SCCP_LIST_LOCK(&hint->subscribers);
 	SCCP_LIST_TRAVERSE(&hint->subscribers, subscriber, list) {
-		if ((d = sccp_device_retain((sccp_device_t *) subscriber->device))) {
-			sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_4 "%s: (sccp_hint_notifySubscribers) notify subscriber %s of %s's state %s (%d)\n", DEV_ID_LOG(d), d->id, (hint->hint_dialplan) ? hint->hint_dialplan : "null", channelstate2str(hint->currentState), hint->currentState);
-
+		AUTO_RELEASE sccp_device_t *d = sccp_device_retain((sccp_device_t *) subscriber->device);
+		if (d) {
+			sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_4 "%s: (sccp_hint_notifySubscribers) notify subscriber %s of %s's state %s (%d)\n", DEV_ID_LOG(d), d->id, (hint->hint_dialplan) ? hint->hint_dialplan : "null", sccp_channelstate2str(hint->currentState), hint->currentState);
 #ifdef CS_DYNAMIC_SPEEDDIAL
 			if (d->inuseprotocolversion >= 15) {
 				sccp_dev_speed_find_byindex((sccp_device_t *) d, subscriber->instance, TRUE, &k);
@@ -1063,7 +1081,7 @@ static void sccp_hint_notifySubscribers(sccp_hint_list_t * hint)
 				   we have dynamic speeddial enabled, but subscriber can not handle this.
 				   We have to switch back to old hint style and send old state.
 				 */
-				sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_4 "%s: (sccp_hint_notifySubscribers) can not handle dynamic speeddial, fall back to old behavior using state %s (%d)\n", DEV_ID_LOG(d), channelstate2str(hint->currentState), hint->currentState);
+				sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_4 "%s: (sccp_hint_notifySubscribers) can not handle dynamic speeddial, fall back to old behavior using state %s (%d)\n", DEV_ID_LOG(d), sccp_channelstate2str(hint->currentState), hint->currentState);
 
 				/*
 				   With the old hint style we should only use SCCP_CHANNELSTATE_ONHOOK and SCCP_CHANNELSTATE_CALLREMOTEMULTILINE as callstate,
@@ -1106,8 +1124,10 @@ static void sccp_hint_notifySubscribers(sccp_hint_list_t * hint)
 					case SCCP_CHANNELSTATE_DND:
 						iconstate = SKINNY_CALLSTATE_CALLREMOTEMULTILINE;
 						break;
+					case sccp_channelstate_LOOKUPERROR:
+						break;
 				}
-				sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_4 "%s: (sccp_hint_notifySubscribers) setting icon to state %s (%d)\n", DEV_ID_LOG(d), channelstate2str(iconstate), iconstate);
+				sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_4 "%s: (sccp_hint_notifySubscribers) setting icon to state %s (%d)\n", DEV_ID_LOG(d), sccp_channelstate2str(iconstate), iconstate);
 
 				if (SCCP_CHANNELSTATE_RINGING == hint->previousState) {
 					/* we send a congestion to the phone, so call will not be marked as missed call */
@@ -1130,7 +1150,6 @@ static void sccp_hint_notifySubscribers(sccp_hint_list_t * hint)
 					sccp_dev_set_keyset(d, subscriber->instance, 0, KEYMODE_INUSEHINT);
 				}
 			}
-			d = sccp_device_release(d);
 		} else {
 			sccp_log((DEBUGCAT_HINT)) (VERBOSE_PREFIX_4 "SCCP: (sccp_hint_notifySubscribers) device not found/retained\n");
 		}
