@@ -8,8 +8,8 @@
  *		Modified by Jan Czmok and Julien Goodwin
  * \note	This program is free software and may be modified and distributed under the terms of the GNU Public License.
  *
- * $Date: 2014-04-14 14:38:49 +0200 (Mon, 14 Apr 2014) $
- * $Revision: 5351 $
+ * $Date: 2014-06-27 00:32:10 +0200 (Fri, 27 Jun 2014) $
+ * $Revision: 5526 $
  */
 
 #include <config.h>
@@ -22,7 +22,7 @@
 #include "sccp_conference.h"
 #include "sccp_actions.h"
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 5351 $")
+SCCP_FILE_VERSION(__FILE__, "$Revision: 5526 $")
 
 static void __sccp_indicate_remote_device(sccp_device_t * device, sccp_channel_t * c, sccp_line_t * line, uint8_t state);
 
@@ -44,39 +44,38 @@ static void __sccp_indicate_remote_device(sccp_device_t * device, sccp_channel_t
  *  - line->devices is not always locked
  * 
  */
-void __sccp_indicate(sccp_device_t * device, sccp_channel_t * c, uint8_t state, uint8_t debug, char *file, int line, const char *pretty_function)
+//void __sccp_indicate(sccp_device_t * device, sccp_channel_t * c, uint8_t state, uint8_t debug, char *file, int line, const char *pretty_function)
+void __sccp_indicate(sccp_device_t * device, sccp_channel_t * c, sccp_channelstate_t state, uint8_t debug, char *file, int line, const char *pretty_function)
 {
-	sccp_device_t *d;
-	sccp_line_t *l;
 	int instance = 0;
-	sccp_linedevices_t *linedevice;
 
 	if (debug) {
 		sccp_log((DEBUGCAT_INDICATE)) (VERBOSE_PREFIX_1 "SCCP: [INDICATE] state '%d' in file '%s', on line %d (%s)\n", state, file, line, pretty_function);
 	}
 
-	d = (device) ? sccp_device_retain(device) : sccp_channel_getDevice_retained(c);
+	AUTO_RELEASE sccp_device_t *d = (device) ? sccp_device_retain(device) : sccp_channel_getDevice_retained(c);
 	if (!d) {
 		sccp_log((DEBUGCAT_INDICATE)) (VERBOSE_PREFIX_1 "SCCP: The channel %d does not have a device\n", c->callid);
 		return;
 	}
 
-	if (!(l = sccp_line_retain(c->line))) {
+	AUTO_RELEASE sccp_line_t *l = sccp_line_retain(c->line);
+	if (!l) {
 		pbx_log(LOG_ERROR, "SCCP: The channel %d does not have a line\n", c->callid);
-		d = sccp_device_release(d);
 		return;
 	}
 
-	if ((linedevice = sccp_linedevice_find(d, l))) {
+	AUTO_RELEASE sccp_linedevices_t *linedevice = sccp_linedevice_find(d, l);
+	if (linedevice) {
 		instance = linedevice->lineInstance;
 	} else {
 		pbx_log(LOG_WARNING, "SCCP: The linedevice/instance for device %s and line %s belonging to channel %d could not be found\n", DEV_ID_LOG(d), l->name, c->callid);
 	}
 
 	/* all the check are ok. We can safely run all the dev functions with no more checks */
-	sccp_log((DEBUGCAT_INDICATE + DEBUGCAT_DEVICE + DEBUGCAT_LINE)) (VERBOSE_PREFIX_3 "%s: Indicate SCCP state %d (%s),channel state %d (%s) on call %s-%08x (previous channelstate %d (%s))\n", d->id, state, sccp_indicate2str(state), c->state, sccp_indicate2str(c->state), l->name, c->callid, c->previousChannelState, sccp_indicate2str(c->previousChannelState));
-	sccp_channel_setSkinnyCallstate(c, state);
-
+	sccp_log((DEBUGCAT_INDICATE + DEBUGCAT_DEVICE + DEBUGCAT_LINE)) (VERBOSE_PREFIX_3 "%s: Indicate SCCP state %d (%s),channel state %d (%s) on call %s-%08x (previous channelstate %d (%s))\n", d->id, state, sccp_channelstate2str(state), c->state, sccp_channelstate2str(c->state), l->name, c->callid, c->previousChannelState, sccp_channelstate2str(c->previousChannelState));
+	sccp_channel_setChannelstate(c, state);
+	
 	switch (state) {
 		case SCCP_CHANNELSTATE_DOWN:
 			//PBX(set_callstate)(c, AST_STATE_DOWN);
@@ -88,7 +87,7 @@ void __sccp_indicate(sccp_device_t * device, sccp_channel_t * c, uint8_t state,
 				PBX(set_callstate) (c, AST_STATE_OFFHOOK);
 				sccp_device_sendcallstate(d, instance, c->callid, SKINNY_CALLSTATE_OFFHOOK, SKINNY_CALLPRIORITY_LOW, SKINNY_CALLINFO_VISIBILITY_DEFAULT);
 				sccp_dev_set_cplane(d, instance, 1);
-				sccp_dev_displayprompt(d, instance, c->callid, SKINNY_DISP_ENTER_NUMBER, 0);
+				sccp_dev_displayprompt(d, instance, c->callid, SKINNY_DISP_ENTER_NUMBER, GLOB(digittimeout));
 				sccp_dev_set_keyset(d, instance, c->callid, KEYMODE_OFFHOOK);
 				sccp_dev_starttone(d, SKINNY_TONE_INSIDEDIALTONE, instance, c->callid, 0);
 			} else {										// call pickup
@@ -106,7 +105,7 @@ void __sccp_indicate(sccp_device_t * device, sccp_channel_t * c, uint8_t state,
 			sccp_dev_set_speaker(d, SKINNY_STATIONSPEAKER_ON);
 			PBX(set_callstate) (c, AST_STATE_OFFHOOK);
 			sccp_device_sendcallstate(d, instance, c->callid, SKINNY_CALLSTATE_OFFHOOK, SKINNY_CALLPRIORITY_LOW, SKINNY_CALLINFO_VISIBILITY_DEFAULT);
-			sccp_dev_displayprompt(d, instance, c->callid, SKINNY_DISP_ENTER_NUMBER, 0);
+			sccp_dev_displayprompt(d, instance, c->callid, SKINNY_DISP_ENTER_NUMBER, GLOB(digittimeout));
 			sccp_dev_set_keyset(d, instance, c->callid, KEYMODE_DIGITSFOLL);
 			sccp_dev_set_cplane(d, instance, 1);
 			sccp_dev_starttone(d, SKINNY_TONE_ZIPZIP, instance, c->callid, 0);
@@ -121,55 +120,23 @@ void __sccp_indicate(sccp_device_t * device, sccp_channel_t * c, uint8_t state,
 			sccp_dev_set_ringer(d, SKINNY_RINGTYPE_OFF, instance, c->callid);
 			sccp_dev_set_speaker(d, SKINNY_STATIONSPEAKER_ON);
 			sccp_device_sendcallstate(d, instance, c->callid, SKINNY_CALLSTATE_OFFHOOK, SKINNY_CALLPRIORITY_LOW, SKINNY_CALLINFO_VISIBILITY_DEFAULT);
-			sccp_dev_displayprompt(d, instance, c->callid, SKINNY_DISP_ENTER_NUMBER, 0);
+			sccp_dev_displayprompt(d, instance, c->callid, SKINNY_DISP_ENTER_NUMBER, GLOB(digittimeout));
 			sccp_dev_set_keyset(d, instance, c->callid, KEYMODE_OFFHOOK);
 			sccp_dev_set_cplane(d, instance, 1);
 			PBX(set_callstate) (c, AST_STATE_OFFHOOK);
 			/* for earlyrtp take a look at sccp_channel_newcall because we have no c->owner here */
 			break;
 		case SCCP_CHANNELSTATE_ONHOOK:
-/*												// reverse order for openreceivechannel check to work
 			c->state = SCCP_CHANNELSTATE_DOWN;
-			PBX(set_callstate) (c, AST_STATE_DOWN);
-*/
-// 			PBX(set_callstate) (c, AST_STATE_DOWN);
-			c->state = SCCP_CHANNELSTATE_DOWN;
-/*			if (c == d->active_channel) {						// prevent setting twice
-				sccp_dev_stoptone(d, instance, c->callid);
-			}
-*/			
-			sccp_dev_stoptone(d, instance, c->callid);
-			sccp_device_setLamp(d, SKINNY_STIMULUS_LINE, instance, SKINNY_LAMP_OFF);
-			sccp_dev_cleardisplaynotify(d);
-			sccp_dev_clearprompt(d, instance, c->callid);
-
-			/** if channel was answered somewhere, set state to connected before onhook -> no missedCalls entry */
-			if (c->answered_elsewhere) {
-				sccp_device_sendcallstate(d, instance, c->callid, SKINNY_CALLSTATE_CONNECTED, SKINNY_CALLPRIORITY_LOW, SKINNY_CALLINFO_VISIBILITY_HIDDEN);
-			}
-
-			sccp_dev_set_ringer(d, SKINNY_RINGTYPE_OFF, instance, c->callid);
-			sccp_device_sendcallstate(d, instance, c->callid, SKINNY_CALLSTATE_ONHOOK, SKINNY_CALLPRIORITY_LOW, SKINNY_CALLINFO_VISIBILITY_DEFAULT);
-			sccp_dev_set_cplane(d, instance, 0);
-			sccp_dev_set_keyset(d, instance, c->callid, KEYMODE_ONHOOK);
-
-			sccp_handle_time_date_req(d->session, d, NULL);	/** we need datetime on hangup for 7936 */
-			if (c == d->active_channel) {
-				sccp_dev_set_speaker(d, SKINNY_STATIONSPEAKER_OFF);
-			}
-
-/*			if (c->previousChannelState == SCCP_CHANNELSTATE_RINGING) {		// prevent setting twice
-				sccp_dev_set_ringer(d, SKINNY_RINGTYPE_OFF, instance, c->callid);
-			}
-*/
+			d->indicate->onhook(d, c, instance, c->callid);
 			break;
 		case SCCP_CHANNELSTATE_RINGOUT:
 			sccp_device_sendcallstate(d, instance, c->callid, SKINNY_CALLSTATE_RINGOUT, SKINNY_CALLPRIORITY_LOW, SKINNY_CALLINFO_VISIBILITY_DEFAULT);
 			d->protocol->sendDialedNumber(d, c);
-			sccp_dev_displayprompt(d, instance, c->callid, SKINNY_DISP_RING_OUT, 0);
+			sccp_dev_displayprompt(d, instance, c->callid, SKINNY_DISP_RING_OUT, GLOB(digittimeout));
 			d->protocol->sendCallInfo(d, c, instance);
 			if (!c->rtp.audio.rtp) {
-				if (d->earlyrtp) {
+				if (d->earlyrtp <= SCCP_EARLYRTP_RINGOUT) {
 					sccp_channel_openReceiveChannel(c);
 				} else {
 					sccp_dev_starttone(d, (uint8_t) SKINNY_TONE_ALERTINGTONE, instance, c->callid, 0);
@@ -217,30 +184,30 @@ void __sccp_indicate(sccp_device_t * device, sccp_channel_t * c, uint8_t state,
 			char prompt[100];
 
 			if (c->ringermode == SKINNY_RINGTYPE_URGENT) {
-				snprintf(prompt, sizeof(prompt), "Urgent Call from: %s", strlen(c->callInfo.callingPartyName) ? c->callInfo.callingPartyName : c->callInfo.callingPartyNumber);
+				snprintf(prompt, sizeof(prompt), "<Urgent>" SKINNY_DISP_FROM ": %s", strlen(c->callInfo.callingPartyName) ? c->callInfo.callingPartyName : c->callInfo.callingPartyNumber);
 			} else {
-				snprintf(prompt, sizeof(prompt), "Incoming Call from: %s", strlen(c->callInfo.callingPartyName) ? c->callInfo.callingPartyName : c->callInfo.callingPartyNumber);
+				snprintf(prompt, sizeof(prompt), SKINNY_DISP_FROM ": %s", strlen(c->callInfo.callingPartyName) ? c->callInfo.callingPartyName : c->callInfo.callingPartyNumber);
 			}
-			sccp_dev_displayprompt(d, instance, c->callid, prompt, 0);
+			sccp_dev_displayprompt(d, instance, c->callid, prompt, GLOB(digittimeout));
 
 			PBX(set_callstate) (c, AST_STATE_RINGING);						/*!\todo thats not the right place to update pbx state */
 			break;
 		case SCCP_CHANNELSTATE_CONNECTED:
-
 			if (linedevice) {
 				d->indicate->connected(d, linedevice, c);
 			}
+			sccp_dev_set_keyset(d, instance, c->callid, KEYMODE_CONNECTED);
 			if (!c->rtp.audio.rtp || c->previousChannelState == SCCP_CHANNELSTATE_HOLD || c->previousChannelState == SCCP_CHANNELSTATE_CALLTRANSFER || c->previousChannelState == SCCP_CHANNELSTATE_CALLCONFERENCE || c->previousChannelState == SCCP_CHANNELSTATE_OFFHOOK) {
 				sccp_channel_openReceiveChannel(c);
 			} else if (c->rtp.audio.rtp) {
-				sccp_log((DEBUGCAT_RTP)) (VERBOSE_PREFIX_3 "%s: Did not reopen an RTP stream as old SCCP state was (%s)\n", d->id, sccp_indicate2str(c->previousChannelState));
+				sccp_log((DEBUGCAT_RTP)) (VERBOSE_PREFIX_3 "%s: Did not reopen an RTP stream as old SCCP state was (%s)\n", d->id, sccp_channelstate2str(c->previousChannelState));
 			}
 			break;
 		case SCCP_CHANNELSTATE_BUSY:
 			if (!c->rtp.audio.rtp) {
 				sccp_dev_starttone(d, SKINNY_TONE_LINEBUSYTONE, instance, c->callid, 0);
 			}
-			sccp_dev_displayprompt(d, instance, c->callid, SKINNY_DISP_BUSY, 0);
+			sccp_dev_displayprompt(d, instance, c->callid, SKINNY_DISP_BUSY, GLOB(digittimeout));
 			PBX(set_callstate) (c, AST_STATE_BUSY);
 			break;
 		case SCCP_CHANNELSTATE_PROGRESS:								/* \todo SCCP_CHANNELSTATE_PROGRESS To be checked */
@@ -250,11 +217,11 @@ void __sccp_indicate(sccp_device_t * device, sccp_channel_t * c, uint8_t state,
 				/** this is a bug of asterisk 1.6 (it sends progress after a call is answered then diverted to some extensions with dial app) */
 				sccp_log((DEBUGCAT_INDICATE)) (VERBOSE_PREFIX_3 "SCCP: Asterisk requests to change state to (Progress) after (Connected). Ignoring\n");
 			} else {
-				sccp_log((DEBUGCAT_INDICATE)) (VERBOSE_PREFIX_3 "SCCP: Asterisk requests to change state to (Progress) from (%s)\n", sccp_indicate2str(c->previousChannelState));
-				if (!c->rtp.audio.rtp && d->earlyrtp) {
+				sccp_log((DEBUGCAT_INDICATE)) (VERBOSE_PREFIX_3 "SCCP: Asterisk requests to change state to (Progress) from (%s)\n", sccp_channelstate2str(c->previousChannelState));
+				if (!c->rtp.audio.rtp && d->earlyrtp <= SCCP_EARLYRTP_PROGRESS) {
 					sccp_channel_openReceiveChannel(c);
 				}
-				sccp_dev_displayprompt(d, instance, c->callid, "Call Progress", 0);
+				sccp_dev_displayprompt(d, instance, c->callid, "Call Progress", GLOB(digittimeout));
 			}
 			break;
 		case SCCP_CHANNELSTATE_PROCEED:
@@ -265,8 +232,8 @@ void __sccp_indicate(sccp_device_t * device, sccp_channel_t * c, uint8_t state,
 			sccp_dev_stoptone(d, instance, c->callid);
 			sccp_device_sendcallstate(d, instance, c->callid, SKINNY_CALLSTATE_PROCEED, SKINNY_CALLPRIORITY_LOW, SKINNY_CALLINFO_VISIBILITY_DEFAULT);	/* send connected, so it is not listed as missed call */
 			d->protocol->sendCallInfo(d, c, instance);
-			sccp_dev_displayprompt(d, instance, c->callid, SKINNY_DISP_CALL_PROCEED, 0);
-			if (!c->rtp.audio.rtp && d->earlyrtp) {
+			sccp_dev_displayprompt(d, instance, c->callid, SKINNY_DISP_CALL_PROCEED, GLOB(digittimeout));
+			if (!c->rtp.audio.rtp && d->earlyrtp <= SCCP_EARLYRTP_RINGOUT) {
 				sccp_channel_openReceiveChannel(c);
 			}
 			break;
@@ -281,7 +248,7 @@ void __sccp_indicate(sccp_device_t * device, sccp_channel_t * c, uint8_t state,
 			sccp_device_setLamp(d, SKINNY_STIMULUS_LINE, instance, SKINNY_LAMP_WINK);
 			sccp_device_sendcallstate(d, instance, c->callid, SKINNY_CALLSTATE_HOLD, SKINNY_CALLPRIORITY_LOW, SKINNY_CALLINFO_VISIBILITY_DEFAULT);	/* send connected, so it is not listed as missed call */
 			sccp_dev_set_keyset(d, instance, c->callid, KEYMODE_ONHOLD);
-			sccp_dev_displayprompt(d, instance, c->callid, SKINNY_DISP_HOLD, 0);
+			sccp_dev_displayprompt(d, instance, c->callid, SKINNY_DISP_HOLD, GLOB(digittimeout));
 			d->protocol->sendCallInfo(d, c, instance);
 			sccp_dev_set_speaker(d, SKINNY_STATIONSPEAKER_OFF);
 			break;
@@ -291,27 +258,38 @@ void __sccp_indicate(sccp_device_t * device, sccp_channel_t * c, uint8_t state,
 				sccp_dev_starttone(d, SKINNY_TONE_REORDERTONE, instance, c->callid, 0);
 			}
 			d->protocol->sendCallInfo(d, c, instance);
-			sccp_dev_displayprompt(d, instance, c->callid, SKINNY_DISP_TEMP_FAIL, 0);
+			sccp_dev_displayprompt(d, instance, c->callid, SKINNY_DISP_TEMP_FAIL, GLOB(digittimeout));
 			// wait 15 seconds, then hangup automatically
 			c->scheduler.hangup = sccp_sched_add(15000, sccp_channel_sched_endcall_by_callid, &c->callid);
 			break;
 		case SCCP_CHANNELSTATE_CALLWAITING:
-			sccp_log((DEBUGCAT_INDICATE)) (VERBOSE_PREFIX_3 "%s: SCCP_CHANNELSTATE_CALLWAITING (%s)\n", DEV_ID_LOG(d), sccp_indicate2str(c->previousChannelState));
-			sccp_channel_callwaiting_tone_interval(d, c);
-			sccp_device_sendcallstate(d, instance, c->callid, SKINNY_CALLSTATE_RINGIN, SKINNY_CALLPRIORITY_LOW, SKINNY_CALLINFO_VISIBILITY_DEFAULT);	/* send connected, so it is not listed as missed call */
-			d->protocol->sendCallInfo(d, c, instance);
-			sccp_dev_displayprompt(d, instance, c->callid, SKINNY_DISP_CALL_WAITING, 0);
-			sccp_dev_set_ringer(d, SKINNY_RINGTYPE_SILENT, instance, c->callid);
-			sccp_dev_set_keyset(d, instance, c->callid, KEYMODE_RINGIN);
-			PBX(set_callstate) (c, AST_STATE_RINGING);
+			{
+				{
+					/* When dialing a shared line which you also have registered, we don't want to outgoing call to show up on our own device as a callwaiting call */
+					AUTO_RELEASE sccp_channel_t *activeChannel = sccp_device_getActiveChannel(d);
+					if (activeChannel && (sccp_strequals(PBX(getChannelLinkedId) (activeChannel), PBX(getChannelLinkedId) (c)))) {
+						sccp_log(DEBUGCAT_INDICATE)(VERBOSE_PREFIX_3 "%s: (SCCP_CHANNELSTATE_CALLWAITING) Already Own Part of the Call: Skipping\n", DEV_ID_LOG(d));
+						sccp_log_and(DEBUGCAT_INDICATE + DEBUGCAT_HIGH)(VERBOSE_PREFIX_3 "%s: LinkedId: %s / %s: LinkedId Remote: %s\n", DEV_ID_LOG(d), PBX(getChannelLinkedId)(c), DEV_ID_LOG(d), PBX(getChannelLinkedId)(activeChannel));
+						break;
+					}
+				}
+				sccp_log((DEBUGCAT_INDICATE)) (VERBOSE_PREFIX_3 "%s: SCCP_CHANNELSTATE_CALLWAITING (%s)\n", DEV_ID_LOG(d), sccp_channelstate2str(c->previousChannelState));
+				sccp_channel_callwaiting_tone_interval(d, c);
+				sccp_device_sendcallstate(d, instance, c->callid, SKINNY_CALLSTATE_RINGIN, SKINNY_CALLPRIORITY_LOW, SKINNY_CALLINFO_VISIBILITY_DEFAULT);	/* send connected, so it is not listed as missed call */
+				d->protocol->sendCallInfo(d, c, instance);
+				sccp_dev_displayprompt(d, instance, c->callid, SKINNY_DISP_CALL_WAITING, GLOB(digittimeout));
+				sccp_dev_set_ringer(d, SKINNY_RINGTYPE_SILENT, instance, c->callid);
+				sccp_dev_set_keyset(d, instance, c->callid, KEYMODE_RINGIN);
+				PBX(set_callstate) (c, AST_STATE_RINGING);
 #ifdef CS_SCCP_CONFERENCE
-			if (d->conferencelist_active) {
-				sccp_conference_hide_list_ByDevice(d);
-			}
+				if (d->conferencelist_active) {
+					sccp_conference_hide_list_ByDevice(d);
+				}
 #endif
+			}
 			break;
 		case SCCP_CHANNELSTATE_CALLTRANSFER:
-			sccp_dev_displayprompt(d, instance, c->callid, SKINNY_DISP_TRANSFER, 0);
+			sccp_dev_displayprompt(d, instance, c->callid, SKINNY_DISP_TRANSFER, GLOB(digittimeout));
 			sccp_dev_set_ringer(d, SKINNY_RINGTYPE_OFF, instance, c->callid);
 			sccp_device_sendcallstate(d, instance, c->callid, SCCP_CHANNELSTATE_CALLTRANSFER, SKINNY_CALLPRIORITY_LOW, SKINNY_CALLINFO_VISIBILITY_DEFAULT);
 			d->protocol->sendCallInfo(d, c, instance);
@@ -321,24 +299,19 @@ void __sccp_indicate(sccp_device_t * device, sccp_channel_t * c, uint8_t state,
 			break;
 		case SCCP_CHANNELSTATE_INVALIDCONFERENCE:
 			/*! \todo SCCP_CHANNELSTATE_INVALIDCONFERENCE To be implemented */
-			sccp_log((DEBUGCAT_INDICATE)) (VERBOSE_PREFIX_3 "%s: SCCP_CHANNELSTATE_INVALIDCONFERENCE (%s)\n", d->id, sccp_indicate2str(c->previousChannelState));
+			sccp_log((DEBUGCAT_INDICATE)) (VERBOSE_PREFIX_3 "%s: SCCP_CHANNELSTATE_INVALIDCONFERENCE (%s)\n", d->id, sccp_channelstate2str(c->previousChannelState));
 			break;
 		case SCCP_CHANNELSTATE_CONNECTEDCONFERENCE:
-			sccp_log((DEBUGCAT_INDICATE)) (VERBOSE_PREFIX_3 "%s: SCCP_CHANNELSTATE_CONNECTEDCONFERENCE (%s)\n", d->id, sccp_indicate2str(c->previousChannelState));
-			sccp_dev_set_ringer(d, SKINNY_RINGTYPE_OFF, instance, c->callid);
-			sccp_dev_set_speaker(d, SKINNY_STATIONSPEAKER_ON);
-			sccp_dev_stoptone(d, instance, c->callid);
-			sccp_device_setLamp(d, SKINNY_STIMULUS_LINE, instance, SKINNY_LAMP_ON);
-			sccp_device_sendcallstate(d, instance, c->callid, SKINNY_CALLSTATE_CONNECTED, SKINNY_CALLPRIORITY_LOW, SKINNY_CALLINFO_VISIBILITY_DEFAULT);
-			d->protocol->sendCallInfo(d, c, instance);
-			sccp_dev_set_cplane(d, instance, 1);
+			sccp_log((DEBUGCAT_INDICATE)) (VERBOSE_PREFIX_3 "%s: SCCP_CHANNELSTATE_CONNECTEDCONFERENCE (%s)\n", d->id, sccp_channelstate2str(c->previousChannelState));
+			if (linedevice) {
+				d->indicate->connected(d, linedevice, c);
+			}
 			sccp_dev_set_keyset(d, instance, c->callid, KEYMODE_CONNCONF);
-			sccp_dev_displayprompt(d, instance, c->callid, SKINNY_DISP_CONNECTED, 0);
 
 			if (!c->rtp.audio.rtp) {
 				sccp_channel_openReceiveChannel(c);
 			} else if (c->rtp.audio.rtp) {
-				sccp_log((DEBUGCAT_RTP)) (VERBOSE_PREFIX_3 "%s: Did not reopen an RTP stream as old SCCP state was (%s)\n", d->id, sccp_indicate2str(c->previousChannelState));
+				sccp_log((DEBUGCAT_RTP)) (VERBOSE_PREFIX_3 "%s: Did not reopen an RTP stream as old SCCP state was (%s)\n", d->id, sccp_channelstate2str(c->previousChannelState));
 			}
 
 			break;
@@ -355,19 +328,15 @@ void __sccp_indicate(sccp_device_t * device, sccp_channel_t * c, uint8_t state,
 			break;
 		case SCCP_CHANNELSTATE_INVALIDNUMBER:
 			/* this is for the earlyrtp. The 7910 does not play tones if a rtp stream is open */
-			sccp_dev_displayprompt(d, instance, c->callid, SKINNY_DISP_UNKNOWN_NUMBER, 0);
+			sccp_dev_displayprompt(d, instance, c->callid, SKINNY_DISP_UNKNOWN_NUMBER, GLOB(digittimeout));
 			sccp_channel_closeAllMediaTransmitAndReceive(d, c);
 			sccp_dev_starttone(d, SKINNY_TONE_REORDERTONE, instance, c->callid, 0);
 			// wait 15 seconds, then hangup automatically
 			c->scheduler.hangup = sccp_sched_add(15000, sccp_channel_sched_endcall_by_callid, &c->callid);
 			break;
 		case SCCP_CHANNELSTATE_DIALING:
-// 			sccp_dev_stoptone(d, instance, c->callid);
-// 			d->protocol->sendDialedNumber(d, c);
-// 			d->protocol->sendCallInfo(d, c, instance);
-// 			sccp_dev_set_keyset(d, instance, c->callid, KEYMODE_DIGITSFOLL);
 			d->indicate->dialing(d, instance, c);
-			if (d->earlyrtp == SCCP_CHANNELSTATE_DIALING && !c->rtp.audio.rtp) {
+			if (d->earlyrtp <= SCCP_EARLYRTP_DIALING && !c->rtp.audio.rtp) {
 				sccp_channel_openReceiveChannel(c);
 			}
 			break;
@@ -394,10 +363,10 @@ void __sccp_indicate(sccp_device_t * device, sccp_channel_t * c, uint8_t state,
 			sccp_dev_set_keyset(d, instance, c->callid, KEYMODE_DIGITSFOLL);
 			break;
 		case SCCP_CHANNELSTATE_BLINDTRANSFER:								/* \todo SCCP_CHANNELSTATE_BLINDTRANSFER To be implemented */
-			sccp_log((DEBUGCAT_INDICATE)) (VERBOSE_PREFIX_3 "%s: SCCP_CHANNELSTATE_BLINDTRANSFER (%s)\n", d->id, sccp_indicate2str(c->previousChannelState));
+			sccp_log((DEBUGCAT_INDICATE)) (VERBOSE_PREFIX_3 "%s: SCCP_CHANNELSTATE_BLINDTRANSFER (%s)\n", d->id, sccp_channelstate2str(c->previousChannelState));
 			break;
 		default:											/* \todo SCCP_CHANNELSTATE:default To be implemented */
-			sccp_log((DEBUGCAT_INDICATE)) (VERBOSE_PREFIX_3 "%s: SCCP_CHANNELSTATE:default  %s (%d) -> %s (%d)\n", d->id, sccp_indicate2str(c->previousChannelState), c->previousChannelState, sccp_indicate2str(c->state), c->state);
+			sccp_log((DEBUGCAT_INDICATE)) (VERBOSE_PREFIX_3 "%s: SCCP_CHANNELSTATE:default  %s (%d) -> %s (%d)\n", d->id, sccp_channelstate2str(c->previousChannelState), c->previousChannelState, sccp_channelstate2str(c->state), c->state);
 			break;
 	}
 
@@ -422,11 +391,7 @@ void __sccp_indicate(sccp_device_t * device, sccp_channel_t * c, uint8_t state,
 		sccp_event_fire(&event);
 	}
 
-	sccp_log((DEBUGCAT_INDICATE + DEBUGCAT_CHANNEL)) (VERBOSE_PREFIX_3 "%s: Finish to indicate state SCCP (%s) on call %s-%08x\n", d->id, sccp_indicate2str(state), l->name, c->callid);
-
-	d = sccp_device_release(d);
-	l = sccp_line_release(l);
-	linedevice = linedevice ? sccp_linedevice_release(linedevice) : NULL;
+	sccp_log((DEBUGCAT_INDICATE + DEBUGCAT_CHANNEL)) (VERBOSE_PREFIX_3 "%s: Finish to indicate state SCCP (%s) on call %s-%08x\n", d->id, sccp_channelstate2str(state), l->name, c->callid);
 }
 
 /*!
@@ -441,21 +406,17 @@ void __sccp_indicate(sccp_device_t * device, sccp_channel_t * c, uint8_t state,
  */
 static void __sccp_indicate_remote_device(sccp_device_t * device, sccp_channel_t * c, sccp_line_t * line, uint8_t state)
 {
-	sccp_device_t *remoteDevice;
-	sccp_channel_t *activeChannel;
 	sccp_channel_t tmpChannel; 												/*!< use this channel to set original called/calling info */
 	int instance;
 	sccp_phonebook_t phonebookRecord = SCCP_PHONEBOOK_NONE;
 
-	//uint32_t privacyStatus=0;
 	if (!c || !line) {
 		return;
 	}
 
-	uint8_t stateVisibility = (!c->privacy) ? SKINNY_CALLINFO_VISIBILITY_DEFAULT : SKINNY_CALLINFO_VISIBILITY_HIDDEN;
-
 	/** \todo move this to channel->privacy */
-	//if (sccp_channel_getDevic(c)) {
+	//uint32_t privacyStatus=0;
+	//if (sccp_channel_getDevice(c)) {
 	//	privacyStatus = sccp_channel_getDevic(c)->privacyFeature.status & SCCP_PRIVACYFEATURE_HINT;
 	//}
 	///* do not display private lines */
@@ -471,49 +432,62 @@ static void __sccp_indicate_remote_device(sccp_device_t * device, sccp_channel_t
 	}
 	sccp_linedevices_t *linedevice;
 	
-	
 	memset(&tmpChannel, 0, sizeof(sccp_channel_t));
 	tmpChannel.callid = c->callid;
-	sccp_copy_string(tmpChannel.callInfo.callingPartyName, c->callInfo.callingPartyName, sizeof(tmpChannel.callInfo.callingPartyName));
-	sccp_copy_string(tmpChannel.callInfo.calledPartyName, c->callInfo.calledPartyName, sizeof(tmpChannel.callInfo.calledPartyName));
-	sccp_copy_string(tmpChannel.callInfo.callingPartyNumber, c->callInfo.callingPartyNumber, sizeof(tmpChannel.callInfo.callingPartyNumber));
-	sccp_copy_string(tmpChannel.callInfo.calledPartyNumber, c->callInfo.calledPartyNumber, sizeof(tmpChannel.callInfo.calledPartyNumber));
-	tmpChannel.calltype = (c->calltype == SKINNY_CALLTYPE_OUTBOUND) ? SKINNY_CALLTYPE_OUTBOUND : SKINNY_CALLTYPE_INBOUND;
+	if (c->privacy || !c->callInfo.presentation) {
+		sccp_copy_string(tmpChannel.callInfo.callingPartyName, SKINNY_DISP_PRIVATE, sizeof(tmpChannel.callInfo.callingPartyName));
+		sccp_copy_string(tmpChannel.callInfo.calledPartyName, SKINNY_DISP_PRIVATE, sizeof(tmpChannel.callInfo.calledPartyName));
+		sccp_copy_string(tmpChannel.callInfo.callingPartyNumber, SKINNY_DISP_PRIVATE, sizeof(tmpChannel.callInfo.callingPartyNumber));
+		sccp_copy_string(tmpChannel.callInfo.calledPartyNumber, SKINNY_DISP_PRIVATE, sizeof(tmpChannel.callInfo.calledPartyNumber));
+	} else {
+		sccp_copy_string(tmpChannel.callInfo.callingPartyName, c->callInfo.callingPartyName, sizeof(tmpChannel.callInfo.callingPartyName));
+		sccp_copy_string(tmpChannel.callInfo.calledPartyName, c->callInfo.calledPartyName, sizeof(tmpChannel.callInfo.calledPartyName));
+		sccp_copy_string(tmpChannel.callInfo.callingPartyNumber, c->callInfo.callingPartyNumber, sizeof(tmpChannel.callInfo.callingPartyNumber));
+		sccp_copy_string(tmpChannel.callInfo.calledPartyNumber, c->callInfo.calledPartyNumber, sizeof(tmpChannel.callInfo.calledPartyNumber));
+	}
+	tmpChannel.calltype = c->calltype;
+	tmpChannel.callInfo.presentation = c->callInfo.presentation;
 	tmpChannel.line = sccp_line_retain(c->line);
-	
 
-	sccp_log((DEBUGCAT_INDICATE)) (VERBOSE_PREFIX_3 "%s: Indicate state %s (%d) on remote devices for channel %s (call %08x)\n", DEV_ID_LOG(device), sccp_indicate2str(state), state, c->designator, c->callid);
+	sccp_log((DEBUGCAT_INDICATE)) (VERBOSE_PREFIX_3 "%s: Indicate state %s (%d) on remote devices for channel %s (call %08x)\n", DEV_ID_LOG(device), sccp_channelstate2str(state), state, c->designator, c->callid);
 	SCCP_LIST_TRAVERSE(&line->devices, linedevice, list) {
 		if (!linedevice->device) {
 			pbx_log(LOG_NOTICE, "Strange to find a linedevice (%p) here without a valid device connected to it !", linedevice);
 			continue;
 		}
 
-		if (device && linedevice->device == device) {
+		if (linedevice->device == device) {
+			// skip self
 			continue;
 		}
 
-		if ((remoteDevice = sccp_device_retain(linedevice->device))) {
-			/* check if we have one part of the remote channel */
-			if ((activeChannel = sccp_device_getActiveChannel(remoteDevice))) {
-				if (sccp_strequals(PBX(getChannelLinkedId) (activeChannel), PBX(getChannelLinkedId) (c))) {
-					stateVisibility = SKINNY_CALLINFO_VISIBILITY_HIDDEN;
-				}
-				sccp_log(DEBUGCAT_INDICATE)(VERBOSE_PREFIX_3 "%s: LinkedId: %s / %s: LinkedId Remote: %s\n", DEV_ID_LOG(device), PBX(getChannelLinkedId)(c), DEV_ID_LOG(remoteDevice), PBX(getChannelLinkedId)(activeChannel))
-				activeChannel = sccp_channel_release(activeChannel);
-			}
-			/* done */
+		/* check if we have one part of the remote channel */
+		AUTO_RELEASE sccp_device_t *remoteDevice = sccp_device_retain(linedevice->device);
+		if (remoteDevice) {
+			uint8_t stateVisibility = (c->privacy || !c->callInfo.presentation) ? SKINNY_CALLINFO_VISIBILITY_HIDDEN : SKINNY_CALLINFO_VISIBILITY_DEFAULT;
 
+ 			/*! \note SKINNY_CALLINFO_VISIBILITY_HIDDEN on old devices: Dirty Hack to prevent showing the call twice (both incoming and outgoing) */
+			stateVisibility = remoteDevice->protocolversion < 17 ? SKINNY_CALLINFO_VISIBILITY_HIDDEN : stateVisibility;
+
+			{
+				AUTO_RELEASE sccp_channel_t *activeChannel = sccp_device_getActiveChannel(linedevice->device);
+				if (activeChannel) {
+					if (sccp_strequals(PBX(getChannelLinkedId) (activeChannel), PBX(getChannelLinkedId) (c))) {
+						sccp_log(DEBUGCAT_INDICATE)(VERBOSE_PREFIX_3 "%s: (indicate_remote_device) Already Own Part of the Call: Hidden\n", DEV_ID_LOG(device));
+						sccp_log_and(DEBUGCAT_INDICATE + DEBUGCAT_HIGH)(VERBOSE_PREFIX_3 "%s: LinkedId: %s / %s: LinkedId Remote: %s\n", DEV_ID_LOG(device), PBX(getChannelLinkedId)(c), DEV_ID_LOG(remoteDevice), PBX(getChannelLinkedId)(activeChannel));
+						continue;
+					}
+				}
+			}
 
 			instance = linedevice->lineInstance;//sccp_device_find_index_for_line(remoteDevice, line->name);
 			switch (state) {
 				case SCCP_CHANNELSTATE_OFFHOOK:
-					/* do nothing here, we will do the offhook simusation in CONNECTED or ONHOOK -MC */
+					/* do nothing here, we will do the offhook simulation in CONNECTED or ONHOOK -MC */
 					break;
 
 				case SCCP_CHANNELSTATE_DOWN:
 				case SCCP_CHANNELSTATE_ONHOOK:
-
 					if (SKINNY_CALLTYPE_INBOUND == c->calltype && c->answered_elsewhere) {
 						switch(phonebookRecord){
 							case SCCP_PHONEBOOK_RECEIVED:
@@ -529,16 +503,13 @@ static void __sccp_indicate_remote_device(sccp_device_t * device, sccp_channel_t
 							break;
 						}
 					}
-
-					
 					sccp_log(DEBUGCAT_INDICATE)(VERBOSE_PREFIX_3 "%s -> %s: indicate remote onhook (instance: %d, callid: %d)\n", DEV_ID_LOG(device), DEV_ID_LOG(remoteDevice), instance, c->callid);
 					remoteDevice->indicate->remoteOnhook(remoteDevice, linedevice, c);
 					break;
 
 				case SCCP_CHANNELSTATE_CONNECTED:
-
-					
 					switch(c->calltype){
+#if 0
 						case SKINNY_CALLTYPE_OUTBOUND:
 							switch(phonebookRecord){
 								case SCCP_PHONEBOOK_RECEIVED:							  
@@ -546,43 +517,28 @@ static void __sccp_indicate_remote_device(sccp_device_t * device, sccp_channel_t
 									remoteDevice->indicate->dialing(remoteDevice, instance, &tmpChannel);
 									remoteDevice->indicate->proceed(remoteDevice, instance, &tmpChannel);
 // 									remoteDevice->indicate->connected(remoteDevice, linedevice, &tmpChannel); /*TODO add source device to phonebook entry */
-						
-									
+
 									sccp_copy_string(tmpChannel.callInfo.originalCalledPartyName, "originalCalledPartyName", sizeof(tmpChannel.callInfo.calledPartyNumber));
 									sccp_copy_string(tmpChannel.callInfo.originalCalledPartyNumber, "originalCalledPartyNumber", sizeof(tmpChannel.callInfo.calledPartyNumber));
 									tmpChannel.callInfo.originalCalledParty_valid = 1;
-									
-// 									char originalCallingPartyName[StationMaxNameSize];							/*!< Original Calling Party Name */
-// 									char originalCallingPartyNumber[StationMaxDirnumSize];							/*!< Original Calling Party ID */
-// 									unsigned int originalCallingParty_valid:1;
+
 									sccp_copy_string(tmpChannel.callInfo.originalCallingPartyName, "originalCalledPartyName", sizeof(tmpChannel.callInfo.calledPartyNumber));
 									sccp_copy_string(tmpChannel.callInfo.originalCallingPartyNumber, "originalCalledPartyNumber", sizeof(tmpChannel.callInfo.calledPartyNumber));
 									tmpChannel.callInfo.originalCallingParty_valid = 1;
 									
-
-// 									char lastRedirectingPartyName[StationMaxNameSize];							/*!< Original Called Party Name */
-// 									char lastRedirectingPartyNumber[StationMaxDirnumSize];							/*!< Original Called Party ID */
-// 									char lastRedirectingVoiceMailbox[StationMaxDirnumSize];							/*!< Last Redirecting VoiceMail Box */
-// 									unsigned int lastRedirectingVoiceMailbox_valid:1;							/*!< TRUE if the name information is valid/present */
-// 									unsigned int lastRedirectingParty_valid:1;
-									
 									sccp_copy_string(tmpChannel.callInfo.lastRedirectingPartyName, "originalCalledPartyName", sizeof(tmpChannel.callInfo.calledPartyNumber));
 									sccp_copy_string(tmpChannel.callInfo.lastRedirectingPartyNumber, "originalCalledPartyNumber", sizeof(tmpChannel.callInfo.calledPartyNumber));
 									tmpChannel.callInfo.lastRedirectingParty_valid = 1;
-// 									tmpChannel.callInfo.originalCdpnRedirectReason;									/*!< Original Called Party Redirect Reason */
-// 									tmpChannel.callInfo.lastRedirectingReason;
 									
 									remoteDevice->indicate->connected(remoteDevice, linedevice, &tmpChannel); /*TODO add source device to phonebook entry */
-									
-									
-									
 								break;
 								case SCCP_PHONEBOOK_MISSED:
 								case SCCP_PHONEBOOK_NONE:
 									//sccp_device_sendcallstate(remoteDevice, instance, c->callid, SKINNY_CALLSTATE_CONNECTED, SKINNY_CALLPRIORITY_LOW, SKINNY_CALLINFO_VISIBILITY_HIDDEN);
 								break;
 							}
-						break;
+							break;
+#endif						
 						case SKINNY_CALLTYPE_INBOUND:
 							switch(phonebookRecord){
 								case SCCP_PHONEBOOK_RECEIVED:							  
@@ -598,24 +554,22 @@ static void __sccp_indicate_remote_device(sccp_device_t * device, sccp_channel_t
 									sccp_device_sendcallstate(remoteDevice, instance, c->callid, SKINNY_CALLSTATE_CONNECTED, SKINNY_CALLPRIORITY_LOW, SKINNY_CALLINFO_VISIBILITY_HIDDEN);
 								break;
 							}
-						break;
+							break;
 						
-						case SKINNY_CALLTYPE_FORWARD:
-						  
-						break;
+						default:						
+							break;
 					}
 
-					sccp_dev_set_ringer(remoteDevice, SKINNY_RINGTYPE_OFF, instance, c->callid);
-					sccp_dev_clearprompt(remoteDevice, instance, c->callid);
+					sccp_dev_set_ringer(remoteDevice, SKINNY_RINGTYPE_OFF, instance, tmpChannel.callid);
+					sccp_dev_clearprompt(remoteDevice, instance, tmpChannel.callid);
 					sccp_device_setLamp(remoteDevice, SKINNY_STIMULUS_LINE, instance, SKINNY_LAMP_ON);
-					sccp_device_sendcallstate(remoteDevice, instance, c->callid, SKINNY_CALLSTATE_CALLREMOTEMULTILINE, SKINNY_CALLPRIORITY_NORMAL, stateVisibility);
-					//sccp_channel_send_callinfo(remoteDevice, c, instance);
+					sccp_device_sendcallstate(remoteDevice, instance, tmpChannel.callid, SKINNY_CALLSTATE_CALLREMOTEMULTILINE, SKINNY_CALLPRIORITY_NORMAL, stateVisibility);
 					remoteDevice->protocol->sendCallInfo(remoteDevice, &tmpChannel, instance);
-					sccp_dev_set_keyset(remoteDevice, instance, c->callid, KEYMODE_ONHOOKSTEALABLE);
+					sccp_dev_set_keyset(remoteDevice, instance, tmpChannel.callid, KEYMODE_ONHOOKSTEALABLE);
 					break;
 
 				case SCCP_CHANNELSTATE_HOLD:
-					remoteDevice->indicate->remoteHold(remoteDevice, instance, c->callid, SKINNY_CALLPRIORITY_NORMAL, stateVisibility);
+					remoteDevice->indicate->remoteHold(remoteDevice, instance, tmpChannel.callid, SKINNY_CALLPRIORITY_NORMAL, stateVisibility);
 					remoteDevice->protocol->sendCallInfo(remoteDevice, &tmpChannel, instance);
 					break;
 
@@ -623,112 +577,9 @@ static void __sccp_indicate_remote_device(sccp_device_t * device, sccp_channel_t
 					break;
 
 			}
-			//sccp_log((DEBUGCAT_INDICATE)) (VERBOSE_PREFIX_3 "%s: Finish Indicating state %s (%d) on remote device %s for channel %s (call %08x)\n", DEV_ID_LOG(device), sccp_indicate2str(state), state, DEV_ID_LOG(remoteDevice), c->designator, c->callid);
-			remoteDevice = sccp_device_release(remoteDevice);
+			sccp_log((DEBUGCAT_INDICATE)) (VERBOSE_PREFIX_3 "%s: Finish Indicating state %s (%d) on remote device %s for channel %s (call %08x)\n", DEV_ID_LOG(device), sccp_channelstate2str(state), state, DEV_ID_LOG(remoteDevice), c->designator, c->callid);
 		}
 	}
 	
 	tmpChannel.line = sccp_line_release(tmpChannel.line);
 }
-
-/*!
- * \brief Convert SCCP Indication/ChannelState 2 String
- * \param state SCCP_CHANNELSTATE_*
- * \return Channel State String
- */
-const char *sccp_indicate2str(uint8_t state)
-{
-	switch (state) {
-		case SCCP_CHANNELSTATE_DOWN:
-			return "Down";
-		case SCCP_CHANNELSTATE_OFFHOOK:
-			return "OffHook";
-		case SCCP_CHANNELSTATE_ONHOOK:
-			return "OnHook";
-		case SCCP_CHANNELSTATE_RINGOUT:
-			return "RingOut";
-		case SCCP_CHANNELSTATE_RINGING:
-			return "Ringing";
-		case SCCP_CHANNELSTATE_CONNECTED:
-			return "Connected";
-		case SCCP_CHANNELSTATE_BUSY:
-			return "Busy";
-		case SCCP_CHANNELSTATE_PROCEED:
-			return "Proceed";
-		case SCCP_CHANNELSTATE_CONGESTION:
-			return "Congestion";
-		case SCCP_CHANNELSTATE_HOLD:
-			return "Hold";
-		case SCCP_CHANNELSTATE_CALLWAITING:
-			return "CallWaiting";
-		case SCCP_CHANNELSTATE_CALLTRANSFER:
-			return "CallTransfer";
-		case SCCP_CHANNELSTATE_CALLPARK:
-			return "CallPark";
-		case SCCP_CHANNELSTATE_CALLREMOTEMULTILINE:
-			return "CallRemoteMultiline";
-		case SCCP_CHANNELSTATE_INVALIDNUMBER:
-			return "InvalidNumber";
-		case SCCP_CHANNELSTATE_DIALING:
-			return "Dialing";
-		case SCCP_CHANNELSTATE_PROGRESS:
-			return "Progress";
-		case SCCP_CHANNELSTATE_SPEEDDIAL:
-			return "SpeedDial";
-		case SCCP_CHANNELSTATE_DIGITSFOLL:
-			return "DigitsFoll";
-		case SCCP_CHANNELSTATE_INVALIDCONFERENCE:
-			return "InvalidConf";
-		case SCCP_CHANNELSTATE_CONNECTEDCONFERENCE:
-			return "ConnConf";
-		case SCCP_CHANNELSTATE_BLINDTRANSFER:
-			return "BlindTransfer";
-		case SCCP_CHANNELSTATE_ZOMBIE:
-			return "Zombie";
-		case SCCP_CHANNELSTATE_DND:
-			return "Dnd";
-		default:
-			return "Unknown";
-	}
-}
-
-/*!
- * \brief Convert SKINNYCall State 2 String
- * \param state SKINNY_CALLSTATE_*
- * \return Call State String
- */
-const char *sccp_callstate2str(uint8_t state)
-{
-	switch (state) {
-		case SKINNY_CALLSTATE_OFFHOOK:
-			return "OffHook";
-		case SKINNY_CALLSTATE_ONHOOK:
-			return "OnHook";
-		case SKINNY_CALLSTATE_RINGOUT:
-			return "RingOut";
-		case SKINNY_CALLSTATE_RINGIN:
-			return "Ringing";
-		case SKINNY_CALLSTATE_CONNECTED:
-			return "Connected";
-		case SKINNY_CALLSTATE_BUSY:
-			return "Busy";
-		case SKINNY_CALLSTATE_PROCEED:
-			return "Proceed";
-		case SKINNY_CALLSTATE_CONGESTION:
-			return "Congestion";
-		case SKINNY_CALLSTATE_HOLD:
-			return "Hold";
-		case SKINNY_CALLSTATE_CALLWAITING:
-			return "CallWaiting";
-		case SKINNY_CALLSTATE_CALLTRANSFER:
-			return "CallTransfer";
-		case SKINNY_CALLSTATE_CALLPARK:
-			return "CallPark";
-		case SKINNY_CALLSTATE_CALLREMOTEMULTILINE:
-			return "CallRemoteMultiline";
-		case SKINNY_CALLSTATE_INVALIDNUMBER:
-			return "InvalidNumber";
-		default:
-			return "Unknown";
-	}
-}
