@@ -1,4 +1,3 @@
-
 /*!
  * \file	sccp_management.c
  * \brief	SCCP Management Class
@@ -6,8 +5,8 @@
  * \note	This program is free software and may be modified and distributed under the terms of the GNU Public License.
  * 		See the LICENSE file at the top of the source tree.
  * 
- * $Date: 2010-11-22 14:09:28 +0100 (Mon, 22 Nov 2010) $
- * $Revision: 2174 $  
+ * $Date: 2014-06-24 13:30:45 +0200 (Tue, 24 Jun 2014) $
+ * $Revision: 5512 $  
  */
 #include <config.h>
 #ifdef CS_SCCP_MANAGER
@@ -22,13 +21,13 @@
 #include "sccp_actions.h"
 #include "sccp_socket.h"
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 2174 $")
+SCCP_FILE_VERSION(__FILE__, "$Revision: 5512 $")
 
 /*
  * Descriptions
  */
-static char management_show_devices_desc[] = "Description: Lists SCCP devices in text format with details on current status.\n" "\n" "DevicelistComplete.\n" "Variables: \n" "  ActionID: <id>	Action ID for this transaction. Will be returned.\n";
-static char management_show_lines_desc[] = "Description: Lists SCCP lines in text format with details on current status.\n" "\n" "LinelistComplete.\n" "Variables: \n" "  ActionID: <id>	Action ID for this transaction. Will be returned.\n";
+static char management_show_devices_desc[] = "Description: Lists SCCP devices in text format with details on current status. (DEPRECATED in favor of SCCPShowDevices)\n" "\n" "DevicelistComplete.\n" "Variables: \n" "  ActionID: <id>	Action ID for this transaction. Will be returned.\n";
+static char management_show_lines_desc[] = "Description: Lists SCCP lines in text format with details on current status. (DEPRECATED in favor of SCCPShowLines)\n" "\n" "LinelistComplete.\n" "Variables: \n" "  ActionID: <id>	Action ID for this transaction. Will be returned.\n";
 static char management_restart_devices_desc[] = "Description: restart a given device\n" "\n" "Variables:\n" "   Devicename: Name of device to restart\n";
 static char management_show_device_add_line_desc[] = "Description: Lists SCCP devices in text format with details on current status.\n" "\n" "DevicelistComplete.\n" "Variables: \n" "  Devicename: Name of device to restart.\n" "  Linename: Name of line";
 static char management_device_update_desc[] = "Description: restart a given device\n" "\n" "Variables:\n" "   Devicename: Name of device\n";
@@ -36,7 +35,7 @@ static char management_device_set_dnd_desc[] = "Description: set dnd on device\n
 static char management_line_fwd_update_desc[] = "Description: update forward status for line\n" "\n" "Variables:\n" "  Devicename: Name of device\n" "  Linename: Name of line\n" "  Forwardtype: type of cfwd (all | busy | noAnswer)\n" "  Disable: yes Disable call forward (optional)\n" "  Number: number to forward calls (optional)";
 static char management_fetch_config_metadata_desc[] = "Description: fetch configuration metadata\n" "\n" "Variables:\n" "  segment: Config Segment Name (if empty returns all segments).\n" "  option: OptionName (if empty returns all options in sement).";
 static char management_startcall_desc[] = "Description: start a new call on a device/line\n" "\n" "Variables:\n" "  Devicename: Name of the Device\n" "  Linename: Name of the line\n" "  number: Number to call";
-static char management_answercall_desc[] = "Description: answer a ringing channel\n" "\n" "Variables:\n" "  Devicename: Name of the Device\n" "  channelId: Id of the channel to pickup\n";
+static char management_answercall_desc[] = "Description: answer a ringing channel (DEPRECATED in favor of SCCPAnswerCall1)\n" "\n" "Variables:\n" "  Devicename: Name of the Device\n" "  channelId: Id of the channel to pickup\n";
 static char management_hangupcall_desc[] = "Description: hangup a channel/call\n" "\n" "Variables:\n" "  channelId: Id of the Channel to hangup\n";
 static char management_hold_desc[] = "Description: hold/resume a call\n" "\n" "Variables:\n" "  channelId: Id of the channel to hold/unhold\n" "  hold: hold=true / resume=false\n" "  Devicename: Name of the Device\n" "  SwapChannels: Swap channels when resuming and an active channel is present (true/false)\n";
 
@@ -68,6 +67,7 @@ static struct manager_custom_hook sccp_manager_hook = {
 
 /*!
  * \brief Register management commands
+ * \note deprecated
  */
 int sccp_register_management(void)
 {
@@ -201,7 +201,7 @@ void sccp_manager_eventListener(const sccp_event_t * event)
 
 			switch (featureType) {
 				case SCCP_FEATURE_DND:
-					manager_event(EVENT_FLAG_CALL, "DND", "ChannelType: SCCP\r\nChannelObjectType: Device\r\nFeature: %s\r\nStatus: %s\r\nSCCPDevice: %s\r\n", featureType2str(SCCP_FEATURE_DND), dndmode2str(device->dndFeature.status), DEV_ID_LOG(device));
+					manager_event(EVENT_FLAG_CALL, "DND", "ChannelType: SCCP\r\nChannelObjectType: Device\r\nFeature: %s\r\nStatus: %s\r\nSCCPDevice: %s\r\n", featureType2str(SCCP_FEATURE_DND), sccp_dndmode2str(device->dndFeature.status), DEV_ID_LOG(device));
 					break;
 				case SCCP_FEATURE_CFWDALL:
 				case SCCP_FEATURE_CFWDBUSY:
@@ -233,6 +233,7 @@ void sccp_manager_eventListener(const sccp_event_t * event)
  * \return Success as int
  * 
  * \called_from_asterisk
+ * \note deprecated
  * 
  */
 static int sccp_manager_show_devices(struct mansession *s, const struct message *m)
@@ -266,10 +267,10 @@ static int sccp_manager_show_devices(struct mansession *s, const struct message
 		astman_append(s, "ObjectType: device\r\n");
 		astman_append(s, "Description: %s\r\n", device->description);
 		astman_append(s, "IPaddress: %s\r\n", clientAddress);
-		astman_append(s, "Reg_Status: %s\r\n", registrationstate2str(device->registrationState));
+		astman_append(s, "Reg_Status: %s\r\n", skinny_registrationstate2str(device->registrationState));
 		astman_append(s, "Reg_Time: %s\r\n", regtime);
 		astman_append(s, "Active: %s\r\n", (device->active_channel) ? "Yes" : "No");
-		astman_append(s, "NumLines: %d\r\n", device->configurationStatistic.numberOfLines);
+		astman_append(s, "NumLines: %d\r\n\r\n", device->configurationStatistic.numberOfLines);
 		total++;
 	}
 
@@ -288,6 +289,7 @@ static int sccp_manager_show_devices(struct mansession *s, const struct message
  * \return Success as int
  * 
  * \called_from_asterisk
+ * \note deprecated
  * 
  */
 static int sccp_manager_show_lines(struct mansession *s, const struct message *m)
@@ -309,7 +311,7 @@ static int sccp_manager_show_lines(struct mansession *s, const struct message *m
 		astman_append(s, "ObjectType: line\r\n");
 		astman_append(s, "Name: %s\r\n", line->name);
 		astman_append(s, "Description: %s\r\n", line->description);
-		astman_append(s, "Num_Channels: %d\r\n", SCCP_RWLIST_GETSIZE(&line->channels));
+		astman_append(s, "Num_Channels: %d\r\n\r\n", SCCP_RWLIST_GETSIZE(&line->channels));
 		total++;
 	}
 
@@ -331,7 +333,6 @@ static int sccp_manager_show_lines(struct mansession *s, const struct message *m
 static int sccp_manager_restart_device(struct mansession *s, const struct message *m)
 {
 	// sccp_list_t *hintList = NULL;
-	sccp_device_t *d = NULL;
 	const char *deviceName = astman_get_header(m, "Devicename");
 	const char *type = astman_get_header(m, "Type");
 
@@ -347,7 +348,7 @@ static int sccp_manager_restart_device(struct mansession *s, const struct messag
 		type = "quick";
 	}
 
-	d = sccp_device_find_byid(deviceName, FALSE);
+	AUTO_RELEASE sccp_device_t *d = sccp_device_find_byid(deviceName, FALSE);
 	if (!d) {
 		astman_send_error(s, m, "Device not found");
 		return 0;
@@ -355,7 +356,6 @@ static int sccp_manager_restart_device(struct mansession *s, const struct messag
 
 	if (!d->session) {
 		astman_send_error(s, m, "Device not registered");
-		d = sccp_device_release(d);
 		return 0;
 	}
 
@@ -368,7 +368,6 @@ static int sccp_manager_restart_device(struct mansession *s, const struct messag
 	astman_send_ack(s, m, "Device restarted");
 	//astman_append(s, "Send %s restart to device %s\r\n", type, deviceName);
 	//astman_append(s, "\r\n");
-	d = sccp_device_release(d);
 
 	return 0;
 }
@@ -383,8 +382,6 @@ static int sccp_manager_restart_device(struct mansession *s, const struct messag
  */
 static int sccp_manager_device_add_line(struct mansession *s, const struct message *m)
 {
-	sccp_device_t *d = NULL;
-	sccp_line_t *line = NULL;
 	const char *deviceName = astman_get_header(m, "Devicename");
 	const char *lineName = astman_get_header(m, "Linename");
 
@@ -400,24 +397,20 @@ static int sccp_manager_device_add_line(struct mansession *s, const struct messa
 		return 0;
 	}
 
-	d = sccp_device_find_byid(deviceName, FALSE);
+	AUTO_RELEASE sccp_device_t *d = sccp_device_find_byid(deviceName, FALSE);
 	if (!d) {
 		astman_send_error(s, m, "Device not found");
 		return 0;
 	}
 
-	line = sccp_line_find_byname(lineName, TRUE);
+	AUTO_RELEASE sccp_line_t *line = sccp_line_find_byname(lineName, TRUE);
 	if (!line) {
 		astman_send_error(s, m, "Line not found");
-		d = sccp_device_release(d);
 		return 0;
 	}
 	sccp_config_addButton(d, -1, LINE, line->name, NULL, NULL);
 	astman_append(s, "Done\r\n");
 	astman_append(s, "\r\n");
-	line = sccp_line_release(line);
-	d = sccp_device_release(d);
-
 	return 0;
 }
 
@@ -431,9 +424,6 @@ static int sccp_manager_device_add_line(struct mansession *s, const struct messa
  */
 static int sccp_manager_line_fwd_update(struct mansession *s, const struct message *m)
 {
-	sccp_line_t *line = NULL;
-	sccp_device_t *d = NULL;
-	sccp_linedevices_t *linedevice = NULL;
 
 	const char *deviceName = astman_get_header(m, "Devicename");
 	const char *lineName = astman_get_header(m, "Linename");
@@ -443,34 +433,29 @@ static int sccp_manager_line_fwd_update(struct mansession *s, const struct messa
 	sccp_callforward_t cfwd_type = SCCP_CFWD_NONE;
 	char cbuf[64] = "";
 
-	d = sccp_device_find_byid(deviceName, TRUE);
+	AUTO_RELEASE sccp_device_t *d = sccp_device_find_byid(deviceName, FALSE);
 	if (!d) {
 		pbx_log(LOG_WARNING, "%s: Device not found\n", deviceName);
 		astman_send_error(s, m, "Device not found");
 		return 0;
 	}
 
-	line = sccp_line_find_byname(lineName, TRUE);
+	AUTO_RELEASE sccp_line_t *line = sccp_line_find_byname(lineName, TRUE);
 	if (!line) {
 		pbx_log(LOG_WARNING, "%s: Line %s not found\n", deviceName, lineName);
 		astman_send_error(s, m, "Line not found");
-		d = sccp_device_release(d);
 		return 0;
 	}
 
 	if (SCCP_LIST_GETSIZE(&line->devices) > 1) {
 		pbx_log(LOG_WARNING, "%s: Callforwarding on shared lines is not supported at the moment\n", deviceName);
 		astman_send_error(s, m, "Callforwarding on shared lines is not supported at the moment");
-		line = sccp_line_release(line);
-		d = sccp_device_release(d);
 		return 0;
 	}
 
 	if (!forwardType) {
 		pbx_log(LOG_WARNING, "%s: Forwardtype is not optional [all | busy]\n", deviceName);
 		astman_send_error(s, m, "Forwardtype is not optional [all | busy]");				/* NoAnswer to be added later on */
-		line = sccp_line_release(line);
-		d = sccp_device_release(d);
 		return 0;
 	}
 
@@ -479,7 +464,8 @@ static int sccp_manager_line_fwd_update(struct mansession *s, const struct messa
 	}
 
 	if (line) {
-		if ((linedevice = sccp_linedevice_find(d, line))) {
+		AUTO_RELEASE sccp_linedevices_t *linedevice = sccp_linedevice_find(d, line);
+		if (linedevice) {
 			if (sccp_strcaseequals("all", forwardType)) {
 				if (sccp_strcaseequals("yes", Disable)) {
 					linedevice->cfwdAll.enabled = 0;
@@ -521,19 +507,13 @@ static int sccp_manager_line_fwd_update(struct mansession *s, const struct messa
 					break;
 			}
 			sccp_dev_forward_status(line, linedevice->lineInstance, linedevice->device);
-			sccp_linedevice_release(linedevice);
 		} else {
 			pbx_log(LOG_WARNING, "%s: LineDevice not found for line %s (Device not registeed ?)\n", deviceName, lineName);
 			astman_send_error(s, m, "LineDevice not found (Device not registered ?)");
-			line = sccp_line_release(line);
-			d = sccp_device_release(d);
 			return 0;
 		}
 	}
 	astman_send_ack(s, m, cbuf);
-
-	line = sccp_line_release(line);
-	d = sccp_device_release(d);
 	return 0;
 }
 
@@ -547,7 +527,6 @@ static int sccp_manager_line_fwd_update(struct mansession *s, const struct messa
  */
 static int sccp_manager_device_update(struct mansession *s, const struct message *m)
 {
-	sccp_device_t *d = NULL;
 	const char *deviceName = astman_get_header(m, "Devicename");
 
 	if (sccp_strlen_zero(deviceName)) {
@@ -555,8 +534,7 @@ static int sccp_manager_device_update(struct mansession *s, const struct message
 		return 0;
 	}
 
-	d = sccp_device_find_byid(deviceName, FALSE);
-
+	AUTO_RELEASE sccp_device_t *d = sccp_device_find_byid(deviceName, FALSE);
 	if (!d) {
 		astman_send_error(s, m, "Device not found");
 		return 0;
@@ -564,7 +542,6 @@ static int sccp_manager_device_update(struct mansession *s, const struct message
 
 	if (!d->session) {
 		astman_send_error(s, m, "Device not active");
-		d = sccp_device_release(d);
 		return 0;
 	}
 
@@ -573,8 +550,6 @@ static int sccp_manager_device_update(struct mansession *s, const struct message
 	sccp_handle_button_template_req(d->session, d, NULL);
 
 	astman_send_ack(s, m, "Done");
-
-	d = sccp_device_release(d);
 	return 0;
 }
 
@@ -588,7 +563,6 @@ static int sccp_manager_device_update(struct mansession *s, const struct message
  */
 static int sccp_manager_device_set_dnd(struct mansession *s, const struct message *m)
 {
-	sccp_device_t *d = NULL;
 	const char *deviceName = astman_get_header(m, "Devicename");
 	const char *DNDState = astman_get_header(m, "DNDState");
 	int prevStatus = 0;
@@ -604,7 +578,8 @@ static int sccp_manager_device_set_dnd(struct mansession *s, const struct messag
 		return 0;
 	}
 	//astman_append(s, "remove channel '%s' from hold\n", channelId);
-	if ((d = sccp_device_find_byid(deviceName, FALSE))) {
+	AUTO_RELEASE sccp_device_t *d = sccp_device_find_byid(deviceName, FALSE);
+	if (d) {
 		if (d->dndFeature.enabled) {
 			prevStatus = d->dndFeature.status;
 			if (sccp_strcaseequals("on", DNDState) || sccp_strcaseequals("reject", DNDState)) {
@@ -618,7 +593,7 @@ static int sccp_manager_device_set_dnd(struct mansession *s, const struct messag
 			}
 
 			if (d->dndFeature.status != prevStatus) {
-				snprintf(retValStr, sizeof(retValStr), "Device %s DND has been set to %s", d->id, dndmode2str(d->dndFeature.status));
+				snprintf(retValStr, sizeof(retValStr), "Device %s DND has been set to %s", d->id, sccp_dndmode2str(d->dndFeature.status));
 				sccp_feat_changed(d, NULL, SCCP_FEATURE_DND);
 				sccp_dev_check_displayprompt(d);
 			} else {
@@ -627,7 +602,6 @@ static int sccp_manager_device_set_dnd(struct mansession *s, const struct messag
 		} else {
 			astman_send_error(s, m, "DND Feature not enabled on this device.");
 		}
-		d = d ? sccp_device_release(d) : NULL;
 	} else {
 		astman_send_error(s, m, "Device could not be found.");
 		return 0;
@@ -647,20 +621,17 @@ static int sccp_manager_device_set_dnd(struct mansession *s, const struct messag
  */
 static int sccp_manager_startCall(struct mansession *s, const struct message *m)
 {
-	sccp_device_t *d = NULL;
-	sccp_line_t *line = NULL;
-	sccp_channel_t *channel = NULL;
-
 	const char *deviceName = astman_get_header(m, "Devicename");
 	const char *lineName = astman_get_header(m, "Linename");
 	const char *number = astman_get_header(m, "number");
 
-	d = sccp_device_find_byid(deviceName, FALSE);
+	AUTO_RELEASE sccp_device_t *d = sccp_device_find_byid(deviceName, FALSE);
 	if (!d) {
 		astman_send_error(s, m, "Device not found");
 		return 0;
 	}
 
+	AUTO_RELEASE sccp_line_t *line = NULL;
 	if (!lineName) {
 		if (d && d->defaultLineInstance > 0) {
 			line = sccp_line_find_byid(d, d->defaultLineInstance);
@@ -673,15 +644,12 @@ static int sccp_manager_startCall(struct mansession *s, const struct message *m)
 
 	if (!line) {
 		astman_send_error(s, m, "Line not found");
-		d = sccp_device_release(d);
 		return 0;
 	}
 
-	channel = sccp_channel_newcall(line, d, sccp_strlen_zero(number) ? NULL : (char *) number, SKINNY_CALLTYPE_OUTBOUND, NULL);
+	AUTO_RELEASE sccp_channel_t *new_channel = NULL;
+	new_channel = sccp_channel_newcall(line, d, sccp_strlen_zero(number) ? NULL : (char *) number, SKINNY_CALLTYPE_OUTBOUND, NULL);
 	astman_send_ack(s, m, "Call Started");
-	line = sccp_line_release(line);
-	d = sccp_device_release(d);
-	channel = channel ? sccp_channel_release(channel) : NULL;
 	return 0;
 }
 
@@ -695,10 +663,8 @@ static int sccp_manager_startCall(struct mansession *s, const struct message *m)
  */
 static int sccp_manager_answerCall(struct mansession *s, const struct message *m)
 {
-	sccp_device_t *d = NULL;
-	sccp_channel_t *c = NULL;
 	char retValStr[64] = "";
-
+ 
 	const char *deviceName = astman_get_header(m, "Devicename");
 	const char *channelId = astman_get_header(m, "channelId");
 
@@ -707,9 +673,16 @@ static int sccp_manager_answerCall(struct mansession *s, const struct message *m
 		astman_send_error(s, m, retValStr);
 		return 0;		
 	}
-	
-	if ((c = sccp_channel_find_byid(atoi(channelId)))) {
-		if ((sccp_strlen_zero(deviceName) && (d = sccp_channel_getDevice_retained(c))) || (!sccp_strlen_zero(deviceName) && (d = sccp_device_find_byid(deviceName, FALSE)))) {
+  	 	  	 		 	
+	AUTO_RELEASE sccp_channel_t *c = sccp_channel_find_byid(atoi(channelId));
+	if (c) {
+		AUTO_RELEASE sccp_device_t *d = NULL;
+		if (sccp_strlen_zero(deviceName)) {
+			d = sccp_channel_getDevice_retained(c);
+		} else {
+			d = sccp_device_find_byid(deviceName, FALSE);
+		}
+		if (d) {
 			if (c->state == SCCP_CHANNELSTATE_RINGING) {
 				sccp_channel_answer(d, c);
 				if (c->owner) {
@@ -720,11 +693,9 @@ static int sccp_manager_answerCall(struct mansession *s, const struct message *m
 			} else {
 				astman_send_error(s, m, "Call is not ringing\r\n");
 			}
-			d = sccp_device_release(d);
 		} else {
 			astman_send_error(s, m, "Device not found");
 		}
-		c = sccp_channel_release(c);
 	} else {
 		astman_send_error(s, m, "Call not found\r\n");
 	}
@@ -741,11 +712,13 @@ static int sccp_manager_answerCall(struct mansession *s, const struct message *m
  */
 static int sccp_manager_hangupCall(struct mansession *s, const struct message *m)
 {
-	sccp_channel_t *c = NULL;
-
 	const char *channelId = astman_get_header(m, "channelId");
+	if (atoi(channelId)==0) {
+		astman_send_error(s, m, "Channel Id has to be a number.");
+		return 0;		
+	}
 
-	c = sccp_channel_find_byid(atoi(channelId));
+	AUTO_RELEASE sccp_channel_t *c = sccp_channel_find_byid(atoi(channelId));
 	if (!c) {
 		astman_send_error(s, m, "Call not found.");
 		return 0;
@@ -753,7 +726,6 @@ static int sccp_manager_hangupCall(struct mansession *s, const struct message *m
 	//astman_append(s, "Hangup call '%s'\r\n", channelId);
 	sccp_channel_endcall(c);
 	astman_send_ack(s, m, "Call was hungup");
-	c = sccp_channel_release(c);
 	return 0;
 }
 
@@ -767,8 +739,6 @@ static int sccp_manager_hangupCall(struct mansession *s, const struct message *m
  */
 static int sccp_manager_holdCall(struct mansession *s, const struct message *m)
 {
-	sccp_channel_t *c = NULL;
-	sccp_device_t *d = NULL;
 	const char *channelId = astman_get_header(m, "channelId");
 	const char *hold = astman_get_header(m, "hold");
 	const char *deviceName = astman_get_header(m, "Devicename");
@@ -776,7 +746,13 @@ static int sccp_manager_holdCall(struct mansession *s, const struct message *m)
 	char *retValStr = "Channel was resumed";
 	boolean_t errorMessage = TRUE;
 
-	c = sccp_channel_find_byid(atoi(channelId));
+	if (atoi(channelId)==0) {
+		snprintf(retValStr, sizeof(retValStr), "Channel Id has to be a number. You have provided: '%s'\r\n", channelId);
+		astman_send_error(s, m, retValStr);
+		return 0;		
+	}
+
+	AUTO_RELEASE sccp_channel_t *c = sccp_channel_find_byid(atoi(channelId));
 	if (!c) {
 		astman_send_error(s, m, "Call not found\r\n");
 		return 0;
@@ -793,8 +769,8 @@ static int sccp_manager_holdCall(struct mansession *s, const struct message *m)
 			retValStr = "To resume a channel, you need to specify the device that resumes call using Devicename variable.";
 			goto SEND_RESPONSE;
 		}
-
-		if ((d = sccp_device_find_byid(deviceName, FALSE))) {
+		AUTO_RELEASE sccp_device_t *d = sccp_device_find_byid(deviceName, FALSE);
+		if (d) {
 			if (sccp_strcaseequals("yes", swap)) {
 				sccp_channel_resume(d, c, TRUE);
 			} else {
@@ -815,9 +791,6 @@ SEND_RESPONSE:
 	} else {
 		astman_send_ack(s, m, retValStr);
 	}
-
-	d = d ? sccp_device_release(d) : NULL;
-	c = c ? sccp_channel_release(c) : NULL;
 	return 0;
 }
 
@@ -867,8 +840,6 @@ static int sccp_asterisk_managerHookHelper(int category, const char *event, char
 	PBX_CHANNEL_TYPE *pbxchannel = NULL;
 	PBX_CHANNEL_TYPE *pbxBridge = NULL;
 
-	sccp_channel_t *channel = NULL;
-	sccp_device_t *d = NULL;
 	char *str, *dupStr;
 
 	if (EVENT_FLAG_CALL == category) {
@@ -884,6 +855,7 @@ static int sccp_asterisk_managerHookHelper(int category, const char *event, char
 			pbx_channel_unlock(pbxchannel);
 #endif
 
+			AUTO_RELEASE sccp_channel_t *channel = NULL;
 			if (pbxchannel && (CS_AST_CHANNEL_PVT_IS_SCCP(pbxchannel))) {
 				channel = get_sccp_channel_from_pbx_channel(pbxchannel);
 			} else if (pbxchannel && ((pbxBridge = pbx_channel_get_by_name(pbx_builtin_getvar_helper(pbxchannel, "BRIDGEPEER"))) != NULL) && (CS_AST_CHANNEL_PVT_IS_SCCP(pbxBridge))) {
@@ -894,7 +866,8 @@ static int sccp_asterisk_managerHookHelper(int category, const char *event, char
 			}
 
 			if (channel) {
-				if ((d = sccp_channel_getDevice_retained(channel))) {
+				AUTO_RELEASE sccp_device_t *d = sccp_channel_getDevice_retained(channel);
+				if (d) {
 					if (!strcasecmp("MonitorStart", event)) {
 						d->monitorFeature.status |= SCCP_FEATURE_MONITOR_STATE_ACTIVE;
 					} else {
@@ -908,9 +881,7 @@ static int sccp_asterisk_managerHookHelper(int category, const char *event, char
 					sccp_dev_send(d, msg_out);
 					
 					sccp_feat_changed(d, NULL, SCCP_FEATURE_MONITOR);
-					d = sccp_device_release(d);
 				}
-				channel = sccp_channel_release(channel);
 			}
 		}
 	}
