@@ -7,8 +7,8 @@
  * \note                This program is free software and may be modified and distributed under the terms of the GNU Public License.
  *              See the LICENSE file at the top of the source tree.
  *
- * $Date$
- * $Revision$
+ * $Date: 2014-06-27 12:33:36 +0200 (Fri, 27 Jun 2014) $
+ * $Revision: 5527 $
  */
 
 #include <config.h>
@@ -20,10 +20,11 @@
 #include "sccp_rtp.h"
 #include "sccp_socket.h"
 
-#define GENERATE_ENUM_STRINGS
+#define ENUMMACRO_FILE "sccp_protocol_enums.hh"
+#define ENUMMACRO_GENERATE ENUMMACRO_GENFUNCTION
 #include "sccp_enum_macro.h"
-#include "sccp_protocol_enums.hh"
-#undef GENERATE_ENUM_STRINGS
+#undef  ENUMMACRO_GENERATE
+#undef  ENUMMACRO_FILE
 
 
 /* CallInfo Message */
@@ -92,7 +93,7 @@ static void sccp_device_sendCallinfoV3(const sccp_device_t * device, const sccp_
 	msg->data.CallInfoMessage.lel_callSecurityStatus = htolel(SKINNY_CALLSECURITYSTATE_UNKNOWN);
 
 	sccp_dev_send(device, msg);
-	sccp_log((DEBUGCAT_CHANNEL | DEBUGCAT_LINE)) (VERBOSE_PREFIX_3 "%s: Send callinfo for %s channel %d on line instance %d" "\n\tcallerid: %s" "\n\tcallerName: %s\n", (device) ? device->id : "(null)", calltype2str(channel->calltype), channel->callid, instance, channel->callInfo.callingPartyNumber, channel->callInfo.callingPartyName);
+	sccp_log((DEBUGCAT_CHANNEL | DEBUGCAT_LINE | DEBUGCAT_INDICATE)) (VERBOSE_PREFIX_3 "%s: Send callinfo(V3) for %s channel %d on line instance %d" "\n\tcallerid: %s" "\n\tcallerName: %s\n", (device) ? device->id : "(null)", skinny_calltype2str(channel->calltype), channel->callid, instance, channel->callInfo.callingPartyNumber, channel->callInfo.callingPartyName);
 }
 
 /*!
@@ -175,6 +176,7 @@ static void sccp_device_sendCallinfoV7(const sccp_device_t * device, const sccp_
 	}
 
 	sccp_dev_send(device, msg);
+	sccp_log((DEBUGCAT_CHANNEL | DEBUGCAT_LINE | DEBUGCAT_INDICATE)) (VERBOSE_PREFIX_3 "%s: Send callinfo(V7) for %s channel %d on line instance %d" "\n\tcallerid: %s" "\n\tcallerName: %s\n", (device) ? device->id : "(null)", skinny_calltype2str(channel->calltype), channel->callid, instance, channel->callInfo.callingPartyNumber, channel->callInfo.callingPartyName);
 }
 
 /*!
@@ -259,6 +261,7 @@ static void sccp_protocol_sendCallinfoV16(const sccp_device_t * device, const sc
 		memcpy(&msg->data.CallInfoDynamicMessage.dummy, &buffer[0], bufferSize);
 	}
 	sccp_dev_send(device, msg);
+	sccp_log((DEBUGCAT_CHANNEL | DEBUGCAT_LINE | DEBUGCAT_INDICATE)) (VERBOSE_PREFIX_3 "%s: Send callinfo(V16) for %s channel %d on line instance %d" "\n\tcallerid: %s" "\n\tcallerName: %s\n", (device) ? device->id : "(null)", skinny_calltype2str(channel->calltype), channel->callid, instance, channel->callInfo.callingPartyNumber, channel->callInfo.callingPartyName);
 }
 
 /* done - callInfoMessage */
@@ -282,7 +285,7 @@ static void sccp_protocol_sendDialedNumberV3(const sccp_device_t * device, const
 	msg->data.DialedNumberMessage.lel_callRef = htolel(channel->callid);
 
 	sccp_dev_send(device, msg);
-	sccp_log(DEBUGCAT_CHANNEL) (VERBOSE_PREFIX_3 "%s: Send the dialed number %s for %s channel %d\n", device->id, channel->callInfo.calledPartyNumber, calltype2str(channel->calltype), channel->callid);
+	sccp_log(DEBUGCAT_CHANNEL) (VERBOSE_PREFIX_3 "%s: Send the dialed number %s for %s channel %d\n", device->id, channel->callInfo.calledPartyNumber, skinny_calltype2str(channel->calltype), channel->callid);
 }
 
 /*!
@@ -305,7 +308,7 @@ static void sccp_protocol_sendDialedNumberV19(const sccp_device_t * device, cons
 	msg->data.DialedNumberMessageV19.lel_callRef = htolel(channel->callid);
 
 	sccp_dev_send(device, msg);
-	sccp_log(DEBUGCAT_CHANNEL) (VERBOSE_PREFIX_3 "%s: Send the dialed number %s for %s channel %d\n", device->id, channel->callInfo.calledPartyNumber, calltype2str(channel->calltype), channel->callid);
+	sccp_log(DEBUGCAT_CHANNEL) (VERBOSE_PREFIX_3 "%s: Send the dialed number %s for %s channel %d\n", device->id, channel->callInfo.calledPartyNumber, skinny_calltype2str(channel->calltype), channel->callid);
 }
 
 /* done - DialedNumber Message */
@@ -565,15 +568,20 @@ static void sccp_protocol_sendOpenReceiveChannelV3(const sccp_device_t * device,
 {
 	int packetSize = 20;											/*! \todo calculate packetSize */
 	sccp_msg_t *msg = sccp_build_packet(OpenReceiveChannel, sizeof(msg->data.OpenReceiveChannel.v3));
-
+	
 	msg->data.OpenReceiveChannel.v3.lel_conferenceId = htolel(channel->callid);
 	msg->data.OpenReceiveChannel.v3.lel_passThruPartyId = htolel(channel->passthrupartyid);
 	msg->data.OpenReceiveChannel.v3.lel_millisecondPacketSize = htolel(packetSize);
 	msg->data.OpenReceiveChannel.v3.lel_payloadType = htolel(channel->rtp.audio.writeFormat);
 	msg->data.OpenReceiveChannel.v3.lel_vadValue = htolel(channel->line->echocancel);
 	msg->data.OpenReceiveChannel.v3.lel_callReference = htolel(channel->callid);
-	msg->data.OpenReceiveChannel.v3.lel_rtptimeout = htolel(10);
 	msg->data.OpenReceiveChannel.v3.lel_remotePortNumber = htolel(4000);
+	if (SCCP_DTMFMODE_SKINNY == channel->dtmfmode) {
+		msg->data.OpenReceiveChannel.v3.lel_RFC2833Type = htolel(0);
+	} else {
+		msg->data.OpenReceiveChannel.v3.lel_RFC2833Type = htolel(SKINNY_CODEC_RFC2833_DYNPAYLOAD);
+	}
+	msg->data.OpenReceiveChannel.v3.lel_dtmfType = htolel(0x0a);
 
 	sccp_dev_send(device, msg);
 }
@@ -592,8 +600,13 @@ static void sccp_protocol_sendOpenReceiveChannelV17(const sccp_device_t * device
 	msg->data.OpenReceiveChannel.v17.lel_payloadType = htolel(channel->rtp.audio.writeFormat);
 	msg->data.OpenReceiveChannel.v17.lel_vadValue = htolel(channel->line->echocancel);
 	msg->data.OpenReceiveChannel.v17.lel_callReference = htolel(channel->callid);
-	msg->data.OpenReceiveChannel.v17.lel_rtptimeout = htolel(10);
 //	msg->data.OpenReceiveChannel.v17.lel_remotePortNumber = htolel(4000);
+	if (SCCP_DTMFMODE_SKINNY == channel->dtmfmode) {
+		msg->data.OpenReceiveChannel.v17.lel_RFC2833Type = htolel(0);
+	} else {
+		msg->data.OpenReceiveChannel.v17.lel_RFC2833Type = htolel(SKINNY_CODEC_RFC2833_DYNPAYLOAD);
+	}
+	msg->data.OpenReceiveChannel.v17.lel_dtmfType = htolel(0x0a);
 
 	struct sockaddr_storage sas;
 //	memcpy(&sas, &device->session->sin, sizeof(struct sockaddr_storage));
@@ -627,8 +640,13 @@ static void sccp_protocol_sendOpenReceiveChannelv22(const sccp_device_t * device
 	msg->data.OpenReceiveChannel.v22.lel_payloadType = htolel(channel->rtp.audio.writeFormat);
 	msg->data.OpenReceiveChannel.v22.lel_vadValue = htolel(channel->line->echocancel);
 	msg->data.OpenReceiveChannel.v22.lel_callReference = htolel(channel->callid);
-	msg->data.OpenReceiveChannel.v22.lel_rtptimeout = htolel(10);
 //	msg->data.OpenReceiveChannel.v22.lel_remotePortNumber = htolel(4000);
+	if (SCCP_DTMFMODE_SKINNY == channel->dtmfmode) {
+		msg->data.OpenReceiveChannel.v22.lel_RFC2833Type = htolel(0);
+	} else {
+		msg->data.OpenReceiveChannel.v22.lel_RFC2833Type = htolel(SKINNY_CODEC_RFC2833_DYNPAYLOAD);
+	}
+	msg->data.OpenReceiveChannel.v22.lel_dtmfType = htolel(0x0a);
 
 	struct sockaddr_storage sas;
 //	memcpy(&sas, &device->session->sin, sizeof(struct sockaddr_storage));
@@ -728,8 +746,13 @@ static void sccp_protocol_sendStartMediaTransmissionV3(const sccp_device_t * dev
 	msg->data.StartMediaTransmission.v3.lel_precedenceValue = htolel(device->audio_tos);
 	msg->data.StartMediaTransmission.v3.lel_ssValue = htolel(channel->line->silencesuppression);		// Silence supression
 	msg->data.StartMediaTransmission.v3.lel_maxFramesPerPacket = htolel(0);
-	msg->data.StartMediaTransmission.v3.lel_rtptimeout = htolel(10);
 	msg->data.StartMediaTransmission.v3.lel_remotePortNumber = htolel(sccp_socket_getPort(&channel->rtp.audio.phone_remote) );
+	if (SCCP_DTMFMODE_SKINNY == channel->dtmfmode) {
+		msg->data.StartMediaTransmission.v3.lel_RFC2833Type = htolel(0);
+	} else {
+		msg->data.StartMediaTransmission.v3.lel_RFC2833Type = htolel(SKINNY_CODEC_RFC2833_DYNPAYLOAD);
+	}
+	msg->data.StartMediaTransmission.v3.lel_dtmfType = htolel(0x0a);
 
 	if(channel->rtp.audio.phone_remote.ss_family == AF_INET){
 		struct sockaddr_in *in = (struct sockaddr_in *)&channel->rtp.audio.phone_remote;
@@ -756,9 +779,14 @@ static void sccp_protocol_sendStartMediaTransmissionV17(const sccp_device_t * de
 	msg->data.StartMediaTransmission.v17.lel_precedenceValue = htolel(device->audio_tos);
 	msg->data.StartMediaTransmission.v17.lel_ssValue = htolel(channel->line->silencesuppression);		// Silence supression
 	msg->data.StartMediaTransmission.v17.lel_maxFramesPerPacket = htolel(0);
-	msg->data.StartMediaTransmission.v17.lel_rtptimeout = htolel(10);
 	msg->data.StartMediaTransmission.v17.lel_remotePortNumber = htolel( sccp_socket_getPort(&channel->rtp.audio.phone_remote) );
-	        
+	if (SCCP_DTMFMODE_SKINNY == channel->dtmfmode) {
+		msg->data.StartMediaTransmission.v17.lel_RFC2833Type = htolel(0);
+	} else {
+		msg->data.StartMediaTransmission.v17.lel_RFC2833Type = htolel(SKINNY_CODEC_RFC2833_DYNPAYLOAD);
+	}
+	msg->data.StartMediaTransmission.v17.lel_dtmfType = htolel(0x0a);
+
 	if (channel->rtp.audio.phone_remote.ss_family == AF_INET6) {
 		struct sockaddr_in6 *in6 = (struct sockaddr_in6 *)&channel->rtp.audio.phone_remote;
 		memcpy(&msg->data.StartMediaTransmission.v17.bel_remoteIpAddr, &in6->sin6_addr, 16);
@@ -786,8 +814,12 @@ static void sccp_protocol_sendStartMediaTransmissionv22(const sccp_device_t * de
 	msg->data.StartMediaTransmission.v22.lel_precedenceValue = htolel(device->audio_tos);
 	msg->data.StartMediaTransmission.v22.lel_ssValue = htolel(channel->line->silencesuppression);		// Silence supression
 	msg->data.StartMediaTransmission.v22.lel_maxFramesPerPacket = htolel(0);
-	msg->data.StartMediaTransmission.v22.lel_rtptimeout = htolel(10);
-	msg->data.StartMediaTransmission.v22.lel_rtpDTMFPayload = htolel(101);
+	if (SCCP_DTMFMODE_SKINNY == channel->dtmfmode) {
+		msg->data.StartMediaTransmission.v22.lel_RFC2833Type = htolel(0);
+	} else {
+		msg->data.StartMediaTransmission.v22.lel_RFC2833Type = htolel(SKINNY_CODEC_RFC2833_DYNPAYLOAD);
+	}
+	msg->data.StartMediaTransmission.v22.lel_dtmfType = htolel(0x0a);
 	msg->data.StartMediaTransmission.v22.lel_remotePortNumber = htolel( sccp_socket_getPort(&channel->rtp.audio.phone_remote) );
 	        
 	if (channel->rtp.audio.phone_remote.ss_family == AF_INET6) {
@@ -1001,7 +1033,6 @@ static void sccp_protocol_parseOpenReceiveChannelAckV3(const sccp_msg_t * msg, s
 	struct sockaddr_in *sin = (struct sockaddr_in *)ss;
 	memcpy(&sin->sin_addr, &msg->data.OpenReceiveChannelAck.v3.bel_ipAddr, INET_ADDRSTRLEN);
 	sin->sin_port = htons(htolel(msg->data.OpenReceiveChannelAck.v3.lel_portNumber));
-	pbx_log(LOG_WARNING, "SCCP: (parseOpenReceiveChannelAckV3) IPv4\n");
 }
 
 static void sccp_protocol_parseOpenReceiveChannelAckV17(const sccp_msg_t * msg, skinny_mediastatus_t * mediastatus, struct sockaddr_storage *ss, uint32_t * passthrupartyid, uint32_t * callReference)
@@ -1015,16 +1046,14 @@ static void sccp_protocol_parseOpenReceiveChannelAckV17(const sccp_msg_t * msg,
 		struct sockaddr_in *sin = (struct sockaddr_in *)ss;
 		memcpy(&sin->sin_addr, &msg->data.OpenReceiveChannelAck.v17.bel_ipAddr, INET_ADDRSTRLEN);
 		sin->sin_port = htons(htolel(msg->data.OpenReceiveChannelAck.v17.lel_portNumber));
- 		pbx_log(LOG_WARNING, "SCCP: (parseOpenReceiveChannelAckV17) IPv4\n");
 	} else {											// read ipv6 address
 		/* what to do with IPv4-mapped IPv6 addresses */
 		ss->ss_family = AF_INET6;
 		struct sockaddr_in6 *sin6 = (struct sockaddr_in6 *)ss;
 		memcpy(&sin6->sin6_addr, &msg->data.OpenReceiveChannelAck.v17.bel_ipAddr, INET6_ADDRSTRLEN);
 		sin6->sin6_port = htons(htolel(msg->data.OpenReceiveChannelAck.v17.lel_portNumber));
- 		pbx_log(LOG_WARNING, "SCCP: (parseOpenReceiveChannelAckV17) IPv6\n");
 	}
-	sccp_dump_msg((sccp_msg_t *)msg);
+	//sccp_dump_msg((sccp_msg_t *)msg);
 }
 
 static void sccp_protocol_parseOpenMultiMediaReceiveChannelAckV3(const sccp_msg_t * msg, skinny_mediastatus_t * mediastatus, struct sockaddr_storage *ss, uint32_t * passthrupartyid, uint32_t * callReference)
@@ -1314,4 +1343,24 @@ const sccp_deviceProtocol_t *sccp_protocol_getDeviceProtocol(const sccp_device_t
 
 	return protocolDef[returnProtocol];
 }
+
+const char *skinny_keymode2longstr(skinny_keymode_t keymode) 
+{
+	switch(keymode) {
+		case KEYMODE_ONHOOK: return "On Hook";
+		case KEYMODE_CONNECTED: return "Connected";
+		case KEYMODE_ONHOLD: return "On Hold";
+		case KEYMODE_RINGIN: return "Incoming Call Ringing";
+		case KEYMODE_OFFHOOK: return "Off Hook";
+		case KEYMODE_CONNTRANS: return "Connect with Transferable Call";
+		case KEYMODE_DIGITSFOLL: return "More Digits Following";
+		case KEYMODE_CONNCONF: return "Connected to Conference";
+		case KEYMODE_RINGOUT: return "Outgoing Call Ringing";
+		case KEYMODE_OFFHOOKFEAT: return "Off Hook with Features";
+		case KEYMODE_INUSEHINT: return "Hint is in use";
+		case KEYMODE_ONHOOKSTEALABLE: return "On Hook with Stealable Call Present";
+		default: return "Unknown KeyMode";
+	}
+}
+
 // kate: indent-width 8; replace-tabs off; indent-mode cstyle; auto-insert-doxygen on; line-numbers on; tab-indents on; keep-extra-spaces off; auto-brackets off;
