@@ -8,8 +8,8 @@
  * \note                This program is free software and may be modified and distributed under the terms of the GNU Public License.
  *              See the LICENSE file at the top of the source tree.
  *
- * $Date: 2014-04-24 02:23:47 +0200 (Thu, 24 Apr 2014) $
- * $Revision: 5384 $
+ * $Date: 2014-06-10 11:52:13 +0200 (Tue, 10 Jun 2014) $
+ * $Revision: 5497 $
  */
 
 #include <netinet/in.h>
@@ -19,7 +19,7 @@
 #include "sccp_device.h"
 #include "sccp_utils.h"
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 5384 $")
+SCCP_FILE_VERSION(__FILE__, "$Revision: 5497 $")
 #ifndef CS_USE_POLL_COMPAT
 #include <poll.h>
 #include <sys/poll.h>
@@ -281,8 +281,8 @@ AST_THREADSTORAGE(sccp_socket_stringify_buf);
 char *sccp_socket_stringify_fmt(const struct sockaddr_storage *sockAddrStorage, int format)
 {
 	const struct sockaddr_storage *sockAddrStorage_tmp;
-	char host[NI_MAXHOST];
-	char port[NI_MAXSERV];
+	char host[NI_MAXHOST] = "";
+	char port[NI_MAXSERV] = "";
 	struct ast_str *str;
 	int e;
 	static const size_t size = sizeof(host) - 1 + sizeof(port) - 1 + 4;
@@ -616,10 +616,7 @@ sccp_device_t *sccp_session_addDevice(sccp_session_t * session, sccp_device_t *
 	if (session && device && session->device != device) {
 		sccp_session_lock(session);
 		if (session->device) {
-			sccp_device_t *remdevice = sccp_device_retain(session->device);
-
 			sccp_session_removeDevice(session);
-			remdevice = sccp_device_release(remdevice);
 		}
 		if ((session->device = sccp_device_retain(device))) {
 			session->device->session = session;
@@ -687,7 +684,6 @@ void sccp_session_close(sccp_session_t * s)
  */
 void destroy_session(sccp_session_t * s, uint8_t cleanupTime)
 {
-	sccp_device_t *d = NULL;
 	boolean_t found_in_list = FALSE;
 	char addrStr[INET6_ADDRSTRLEN];
 
@@ -704,12 +700,14 @@ void destroy_session(sccp_session_t * s, uint8_t cleanupTime)
 	}
 
 	/* cleanup device if this session is not a crossover session */
-	if (s->device && (d = sccp_device_retain(s->device))) {
-		sccp_log((DEBUGCAT_SOCKET)) (VERBOSE_PREFIX_3 "%s: Destroy Device Session %s\n", DEV_ID_LOG(s->device), addrStr);
-		d->registrationState = SKINNY_DEVICE_RS_NONE;
-		d->needcheckringback = 0;
-		sccp_dev_clean(d, (d->realtime) ? TRUE : FALSE, cleanupTime);
-		sccp_device_release(d);
+	if (s->device) {
+		AUTO_RELEASE sccp_device_t *d = sccp_device_retain(s->device);
+		if (d) {
+			sccp_log((DEBUGCAT_SOCKET)) (VERBOSE_PREFIX_3 "%s: Destroy Device Session %s\n", DEV_ID_LOG(s->device), addrStr);
+			d->registrationState = SKINNY_DEVICE_RS_NONE;
+			d->needcheckringback = 0;
+			sccp_dev_clean(d, (d->realtime) ? TRUE : FALSE, cleanupTime);
+		}
 	}
 
 	sccp_log((DEBUGCAT_SOCKET)) (VERBOSE_PREFIX_3 "SCCP: Destroy Session %s\n", addrStr);
