@@ -9,8 +9,8 @@
  * \note	This program is free software and may be modified and distributed under the terms of the GNU Public License.
  *		See the LICENSE file at the top of the source tree.
  *
- * $Date: 2014-04-22 18:27:10 +0200 (Tue, 22 Apr 2014) $
- * $Revision: 5356 $
+ * $Date: 2014-06-27 00:32:10 +0200 (Fri, 27 Jun 2014) $
+ * $Revision: 5526 $
  */
 
 #include <config.h>
@@ -25,7 +25,7 @@
 #include "sccp_actions.h"
 #include "sccp_rtp.h"
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 5356 $")
+SCCP_FILE_VERSION(__FILE__, "$Revision: 5526 $")
 
 /* private prototypes */
 void sccp_sk_videomode(sccp_device_t * d, sccp_line_t * l, const uint32_t lineInstance, sccp_channel_t * c);
@@ -94,7 +94,7 @@ static const struct sccp_softkeyMap_cb softkeyCbMap[] = {
 	{SKINNY_LBL_DND, sccp_sk_dnd, FALSE},
 	{SKINNY_LBL_DIRTRFR, sccp_sk_dirtrfr, TRUE},
 	{SKINNY_LBL_SELECT, sccp_sk_select, TRUE},
-	{SKINNY_LBL_PRIVATE, sccp_sk_private, TRUE},
+	{SKINNY_LBL_PRIVATE, sccp_sk_private, FALSE},
 	{SKINNY_LBL_MONITOR, sccp_feat_monitor, TRUE},
 	{SKINNY_LBL_INTRCPT, sccp_sk_resume, TRUE},
 	{SKINNY_LBL_DIAL, sccp_sk_dial, TRUE},
@@ -140,7 +140,7 @@ const sccp_softkeyMap_cb_t *sccp_getSoftkeyMap_by_SoftkeyEvent(uint32_t event)
 void sccp_softkey_setSoftkeyState(sccp_device_t * device, uint8_t softKeySet, uint8_t softKey, boolean_t enable)
 {
 	uint8_t i;
-	sccp_log((DEBUGCAT_SOFTKEY)) (VERBOSE_PREFIX_3 "%s: softkey '%s' on %s to %s\n", DEV_ID_LOG(device), label2str(softKey), keymode2str(softKeySet), enable ? "on" : "off");
+	sccp_log((DEBUGCAT_SOFTKEY)) (VERBOSE_PREFIX_3 "%s: softkey '%s' on %s to %s\n", DEV_ID_LOG(device), label2str(softKey), skinny_keymode2str(softKeySet), enable ? "on" : "off");
 
 	if (!device) {
 		return;
@@ -173,7 +173,7 @@ void sccp_softkey_setSoftkeyState(sccp_device_t * device, uint8_t softKeySet, ui
 void sccp_sk_dial(sccp_device_t * d, sccp_line_t * l, const uint32_t lineInstance, sccp_channel_t * c)
 {
 	sccp_log((DEBUGCAT_SOFTKEY)) (VERBOSE_PREFIX_3 "%s: SoftKey Dial Pressed\n", DEV_ID_LOG(d));
-	if (c) {												// Handle termination of dialling if in appropriate state.
+	if (c && !PBX(getChannelPbx)(c)) {								// Prevent dialling if in an inappropriate state.
 		/* Only handle this in DIALING state. AFAIK GETDIGITS is used only for call forward and related input functions. (-DD) */
 		if (c->ss_action == SCCP_SS_GETFORWARDEXTEN) {
 			c->scheduler.digittimeout = SCCP_SCHED_DEL(c->scheduler.digittimeout);
@@ -229,8 +229,18 @@ void sccp_sk_redial(sccp_device_t * d, sccp_line_t * l, const uint32_t lineInsta
 	}
 
 #ifdef CS_ADV_FEATURES
+	char *data;
 	if (d->useRedialMenu) {
-		char *data = "<CiscoIPPhoneExecute><ExecuteItem Priority=\"0\" URL=\"Key:Directories\"/><ExecuteItem Priority=\"0\" URL=\"Key:KeyPad3\"/></CiscoIPPhoneExecute>";
+		if (d->session->protocolType == SCCP_PROTOCOL) {
+			if (d->protocolversion < 15) {
+				data = "<CiscoIPPhoneExecute><ExecuteItem Priority=\"0\" URL=\"Key:Directories\"/><ExecuteItem Priority=\"0\" URL=\"Key:KeyPad3\"/></CiscoIPPhoneExecute>";
+			} else {
+				data = "<CiscoIPPhoneExecute><ExecuteItem Priority=\"0\" URL=\"Application:Cisco/PlacedCalls\"/></CiscoIPPhoneExecute>";
+			}
+		} else {
+			data = "<CiscoIPPhoneExecute><ExecuteItem Priority=\"0\" URL=\"Key:Setup\"/><ExecuteItem Priority=\"0\" URL=\"Key:KeyPad1\"/><ExecuteItem Priority=\"0\" URL=\"Key:KeyPad3\"/></CiscoIPPhoneExecute>";
+//			data = "<CiscoIPPhoneExecute><ExecuteItem Priority=\"0\" URL=\"Application:Cisco/PlacedCalls\"/></CiscoIPPhoneExecute>";
+		}
 
 		d->protocol->sendUserToDeviceDataVersionMessage(d, 0, lineInstance, 0, 0, data, 0);
 		return;
@@ -260,9 +270,8 @@ void sccp_sk_redial(sccp_device_t * d, sccp_line_t * l, const uint32_t lineInsta
 			line = sccp_dev_get_activeline(d);
 		}
 		if (line) {
-			if ((c = sccp_channel_newcall(line, d, d->lastNumber, SKINNY_CALLTYPE_OUTBOUND, NULL))) {
-				sccp_channel_release(c);
-			}
+			AUTO_RELEASE sccp_channel_t *new_channel = NULL;
+			new_channel = sccp_channel_newcall(line, d, d->lastNumber, SKINNY_CALLTYPE_OUTBOUND, NULL);
 			line = sccp_line_release(line);
 		} else {
 			sccp_log((DEBUGCAT_SOFTKEY)) (VERBOSE_PREFIX_3 "%s: Redial pressed on a device without a registered line\n", d->id);
@@ -328,9 +337,8 @@ void sccp_sk_newcall(sccp_device_t * d, sccp_line_t * l, const uint32_t lineInst
 		}
 		/* done */
 		
-		c = sccp_channel_newcall(line, d, adhocNumber, SKINNY_CALLTYPE_OUTBOUND, NULL);
-		c = c ? sccp_channel_release(c) : NULL;
-		
+		AUTO_RELEASE sccp_channel_t *new_channel = NULL;
+		new_channel = sccp_channel_newcall(line, d, adhocNumber, SKINNY_CALLTYPE_OUTBOUND, NULL);
 		line = sccp_line_release(line);
 	}
 }
@@ -406,7 +414,7 @@ void sccp_sk_endcall(sccp_device_t * d, sccp_line_t * l, const uint32_t lineInst
 	
 	if (c->calltype == SKINNY_CALLTYPE_INBOUND && 1 < c->subscribers--) { // && pbx_channel_state(c->owner) != AST_STATE_UP) {
 		if (d && d->indicate && d->indicate->onhook) {
-			d->indicate->onhook(d, lineInstance, c->callid);
+			d->indicate->onhook(d, c, lineInstance, c->callid);
 		}
 	} else {
 		sccp_channel_endcall(c);
@@ -430,7 +438,7 @@ void sccp_sk_dnd(sccp_device_t * d, sccp_line_t * l, const uint32_t lineInstance
 		return;
 	}
 
-	sccp_log((DEBUGCAT_SOFTKEY)) (VERBOSE_PREFIX_3 "%s: SoftKey DND Pressed (Current Status: %s, Feature enabled: %s)\n", DEV_ID_LOG(d), dndmode2str(d->dndFeature.status), d->dndFeature.enabled ? "YES" : "NO");
+	sccp_log((DEBUGCAT_SOFTKEY)) (VERBOSE_PREFIX_3 "%s: SoftKey DND Pressed (Current Status: %s, Feature enabled: %s)\n", DEV_ID_LOG(d), sccp_dndmode2str(d->dndFeature.status), d->dndFeature.enabled ? "YES" : "NO");
 
 	if (!d->dndFeature.enabled) {
 		sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "%s: SoftKey DND Feature disabled\n", DEV_ID_LOG(d));
@@ -472,7 +480,7 @@ void sccp_sk_dnd(sccp_device_t * d, sccp_line_t * l, const uint32_t lineInstance
 
 	sccp_feat_changed(d, NULL, SCCP_FEATURE_DND);								/* notify the modules the the DND-feature changed state */
 	sccp_dev_check_displayprompt(d);									/*! \todo we should use the feature changed event to check displayprompt */
-	sccp_log((DEBUGCAT_SOFTKEY)) (VERBOSE_PREFIX_3 "%s: SoftKey DND Pressed (New Status: %s, Feature enabled: %s)\n", DEV_ID_LOG(d), dndmode2str(d->dndFeature.status), d->dndFeature.enabled ? "YES" : "NO");
+	sccp_log((DEBUGCAT_SOFTKEY)) (VERBOSE_PREFIX_3 "%s: SoftKey DND Pressed (New Status: %s, Feature enabled: %s)\n", DEV_ID_LOG(d), sccp_dndmode2str(d->dndFeature.status), d->dndFeature.enabled ? "YES" : "NO");
 }
 
 /*!
@@ -489,7 +497,11 @@ void sccp_sk_backspace(sccp_device_t * d, sccp_line_t * l, const uint32_t lineIn
 	sccp_log((DEBUGCAT_SOFTKEY)) (VERBOSE_PREFIX_3 "%s: SoftKey Backspace Pressed\n", DEV_ID_LOG(d));
 	int len;
 
-	if ((c->state != SCCP_CHANNELSTATE_DIALING) && (c->state != SCCP_CHANNELSTATE_DIGITSFOLL) && (c->state != SCCP_CHANNELSTATE_OFFHOOK)) {
+	if (	((c->state != SCCP_CHANNELSTATE_DIALING) && 
+		(c->state != SCCP_CHANNELSTATE_DIGITSFOLL) && 
+		(c->state != SCCP_CHANNELSTATE_OFFHOOK) &&
+		(c->state != SCCP_CHANNELSTATE_GETDIGITS)) ||
+		PBX(getChannelPbx)(c)) {
 		return;
 	}
 
@@ -539,12 +551,16 @@ void sccp_sk_answer(sccp_device_t * d, sccp_line_t * l, const uint32_t lineInsta
 		return;
 	}
 	sccp_log((DEBUGCAT_SOFTKEY)) (VERBOSE_PREFIX_3 "%s: SoftKey Answer Pressed, instance: %d\n", DEV_ID_LOG(d), lineInstance);
+
+	/* Changed Lock into a Barrier, Lock should actually be moved to sccp_channel_answer if necessary. 
+	   In sccp_channel_answer, we should unlock while lock before changing/updating codecs to prevent a deadlock against setWriteFormat 
+	*/
 	if (c->owner) {
+		/* memmory barrier */
 		pbx_channel_lock(c->owner);
-	}
-	sccp_channel_answer(d, c);
-	if (c->owner) {
 		pbx_channel_unlock(c->owner);
+		
+		sccp_channel_answer(d, c);
 	}
 }
 
@@ -557,14 +573,15 @@ void sccp_sk_answer(sccp_device_t * d, sccp_line_t * l, const uint32_t lineInsta
  * \param c SCCP Channel
  *
  */
-void sccp_sk_dirtrfr(sccp_device_t * d, sccp_line_t * l, const uint32_t lineInstance, sccp_channel_t * c)
+void sccp_sk_dirtrfr(sccp_device_t * device, sccp_line_t * l, const uint32_t lineInstance, sccp_channel_t * c)
 {
-	sccp_log((DEBUGCAT_SOFTKEY)) (VERBOSE_PREFIX_3 "%s: SoftKey Direct Transfer Pressed\n", DEV_ID_LOG(d));
+	sccp_log((DEBUGCAT_SOFTKEY)) (VERBOSE_PREFIX_3 "%s: SoftKey Direct Transfer Pressed\n", DEV_ID_LOG(device));
 
+	sccp_device_t *d = sccp_device_retain(device);
 	sccp_selectedchannel_t *x;
 	sccp_channel_t *chan1 = NULL, *chan2 = NULL, *tmp = NULL;
 
-	if (!(d = sccp_device_retain(d))) {
+	if (!d) {
 		return;
 	}
 	if ((sccp_device_selectedchannels_count(d)) != 2) {
@@ -811,8 +828,10 @@ void sccp_sk_trnsfvm(sccp_device_t * d, sccp_line_t * l, const uint32_t lineInst
  * \param c SCCP Channel
  *
  */
-void sccp_sk_private(sccp_device_t * d, sccp_line_t * l, const uint32_t lineInstance, sccp_channel_t * c)
+void sccp_sk_private(sccp_device_t * d, sccp_line_t * line, const uint32_t lineInstance, sccp_channel_t * channel)
 {
+	AUTO_RELEASE sccp_channel_t *c;	
+	uint8_t instance;
 	if (!d) {
 		sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "SCCP: sccp_sk_private function called without specifying a device\n");
 		return;
@@ -826,19 +845,46 @@ void sccp_sk_private(sccp_device_t * d, sccp_line_t * l, const uint32_t lineInst
 		return;
 	}
 
+	if (channel) {
+		c = sccp_channel_retain(channel);
+		instance = lineInstance;
+	} else {
+		AUTO_RELEASE sccp_line_t *l;
+		sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "%s: Creating new PRIVATE channel\n", d->id);
+		if (line) {
+			l = sccp_line_retain(line);
+		} else {
+			instance = (d->defaultLineInstance > 0) ? d->defaultLineInstance : SCCP_FIRST_LINEINSTANCE;
+			l = sccp_line_find_byid(d, instance);
+		}
+		if (!l) {
+			sccp_dev_displayprompt(d, lineInstance, 0, "PRIVATE with no line active", 5);
+			return;
+		}
+		instance = sccp_device_find_index_for_line(d, l->name);
+		sccp_dev_set_activeline(d, l);
+		sccp_dev_set_cplane(d, instance, 1);
+		c = sccp_channel_newcall(l, d, NULL, SKINNY_CALLTYPE_OUTBOUND, NULL);
+	}
+	
 	if (!c) {
 		sccp_dev_displayprompt(d, lineInstance, 0, "PRIVATE with no channel active", 5);
 		return;
 	}
 
+	// check device->privacyFeature.status before toggeling
+
+	// toggle
 	c->privacy = (c->privacy) ? FALSE : TRUE;
-	//d->privacyFeature.status = c->privacy;			// should not be activeted on softkey
+	
+	// update device->privacyFeature.status  using sccp_feat_changed
 	//sccp_feat_changed(d, NULL, SCCP_FEATURE_PRIVACY);
+	
 	if (c->privacy) {
-		sccp_dev_displayprompt(d, lineInstance, c->callid, SKINNY_DISP_PRIVATE, 0);
+		sccp_dev_displayprompt(d, instance, c->callid, SKINNY_DISP_PRIVATE, 0);
 		c->callInfo.presentation = 0;
 	} else {
-		sccp_dev_displayprompt(d, lineInstance, c->callid, SKINNY_DISP_ENTER_NUMBER, 1);
+		sccp_dev_displayprompt(d, instance, c->callid, SKINNY_DISP_ENTER_NUMBER, 1);
 	}
 	sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "%s: Private %s on call %d\n", d->id, c->privacy ? "enabled" : "disabled", c->callid);
 }
