@@ -8,8 +8,8 @@
  * \note	This program is free software and may be modified and distributed under the terms of the GNU Public License.
  * 		See the LICENSE file at the top of the source tree.
  *
- * $Date: 2014-04-24 02:14:56 +0200 (Thu, 24 Apr 2014) $
- * $Revision: 5377 $
+ * $Date: 2014-06-24 18:22:23 +0200 (Tue, 24 Jun 2014) $
+ * $Revision: 5517 $
  */
 
 #include <config.h>
@@ -18,7 +18,7 @@
 #include "sccp_utils.h"
 #include "sccp_socket.h"
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 5377 $")
+SCCP_FILE_VERSION(__FILE__, "$Revision: 5517 $")
 
 /*!
  * \brief Print out a messagebuffer
@@ -198,7 +198,7 @@ gcc_inline size_t msgtype2size(sccp_mid_t type)
 	return sccp_messagetypes[type].size + SCCP_PACKET_HEADER;
 }
 
-gcc_inline const char *pbxdevicestate2str(uint32_t value)
+gcc_inline const char *pbxsccp_devicestate2str(uint32_t value)
 {														/* pbx_impl/ast/ast.h */
 	_ARR2STR(pbx_devicestates, devicestate, value, text);
 }
@@ -1786,3 +1786,16 @@ int sccp_strversioncmp(const char *s1, const char *s2)
 
 	return strcmp(s1, s2);
 }
+
+char *sccp_dec2binstr(char *buf, size_t size, int value) 
+{
+	char b[33] = {0};
+	int pos;
+	long long z;
+	for ( z = 1LL<<31, pos=0; z>0; z>>=1, pos++)
+	{
+		b[pos] = ( ((value & z) == z) ? '1' : '0');
+	}
+	snprintf(buf, size, "%s", b);
+	return buf;
+}
